@@ -1,13 +1,12 @@
 /**
- ******************************************************************************
+ * \file    client/config_init.h
  *
- * @file    config_init.h
- *
- * @author  Damien Stuart
- *
- * @brief   Header file for fwknop config_init.
- *
- * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief   Header file for fwknop config_init.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -43,6 +42,10 @@
 void config_init(fko_cli_options_t *options, int argc, char **argv);
 void usage(void);
 
+#ifdef HAVE_C_UNIT_TESTS
+int register_ts_config_init(void);
+#endif
+
 #endif /* CONFIG_INIT_H */
 
 /***EOF***/
