@@ -1,11 +1,12 @@
 /**
- * @file    fwknop.c
+ * \file    client/fwknop.c
  *
- * @author  Damien S. Stuart
- *
- * @brief   An implementation of an fwknop client.
- *
- * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief   The fwknop client.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -52,94 +53,28 @@ static int set_nat_access(fko_ctx_t ctx, fko_cli_options_t *options,
 static int set_access_buf(fko_ctx_t ctx, fko_cli_options_t *options,
         char *access_buf);
 static int get_rand_port(fko_ctx_t ctx);
+int resolve_ip_https(fko_cli_options_t *options);
 int resolve_ip_http(fko_cli_options_t *options);
 static void clean_exit(fko_ctx_t ctx, fko_cli_options_t *opts,
     char *key, int *key_len, char *hmac_key, int *hmac_key_len,
     unsigned int exit_status);
 static void zero_buf_wrapper(char *buf, int len);
-static int is_hostname_str_with_port(const char *str,
-        char *hostname, size_t hostname_bufsize, int *port);
+#if HAVE_LIBFIU
+static int enable_fault_injections(fko_cli_options_t * const opts);
+#endif
+
+#if AFL_FUZZING
+  /* These are used in AFL fuzzing mode so the fuzzing cycle is not
+   * interrupted by trying to read from stdin
+  */
+  #define AFL_ENC_KEY               "aflenckey"
+  #define AFL_HMAC_KEY              "aflhmackey"
+#endif
 
-#define MAX_CMDLINE_ARGS            50                  /*!< should be way more than enough */
 #define NAT_ACCESS_STR_TEMPLATE     "%s,%d"             /*!< Template for a nat access string ip,port with sscanf*/
 #define HOSTNAME_BUFSIZE            64                  /*!< Maximum size of a hostname string */
 #define CTX_DUMP_BUFSIZE            4096                /*!< Maximum size allocated to a FKO context dump */
 
-/**
- * @brief Check whether a string is an ipv6 address or not
- *
- * @param str String to check for an ipv6 address.
- *
- * @return 1 if the string is an ipv6 address, 0 otherwise.
- */
-static int
-is_ipv6_str(char *str)
-{
-    return 0;
-}
-
-/**
- * @brief Check a string to find out if it is built as 'hostname,port'
- *
- * This function check if we can extract an hostname and a port from the string.
- * If yes, we return 1, and both the hostname buffer and the port number are set
- * accordingly.
- *
- * We could have used sscanf() here with a template "%[^,],%u", but this way we
- * do not limit the size of the value copy in the hostname destination buffer.
- * Limiting the string in the sscanf() can be done but would prevent any easy change
- * for the hostname buffer size.
- *
- * @param str String to parse.
- * @param hostname Buffer where to store the hostname value read from @str.
- * @param hostname_bufsize Hostname buffer size.
- * @param port Value of the port read from @str.
- *
- * @return 1 if the string is built as 'hostname,port', 0 otherwise.
- */
-static int
-is_hostname_str_with_port(const char *str, char *hostname, size_t hostname_bufsize, int *port)
-{
-    int     valid = 0;                /* Result of the function */
-    char    buf[MAX_LINE_LEN] = {0};  /* Copy of the buffer eg. "hostname,port" */
-    char   *h;                        /* Pointer on the hostname string */
-    char   *p;                        /* Ponter on the port string */
-
-    memset(hostname, 0, hostname_bufsize);
-    *port = 0;
-
-    /* Replace the comma in the string with a NULL char to split the
-     * buffer in two strings (hostname and port) */
-    strlcpy(buf, str, sizeof(buf));
-    p = strchr(buf, ',');
-
-    if(p != NULL)
-    {
-        *p++ = 0;
-        h = buf;
-
-        *port = atoi(p);
-
-        /* If the string does not match an ipv4 or ipv6 address we assume this
-         * is an hostname. We make sure the port is in the good range too */
-        if (   (is_valid_ipv4_addr(buf) == 0)
-            && (is_ipv6_str(buf) == 0)
-            && ((*port > 0) && (*port < 65536)) )
-        {
-            strlcpy(hostname, h, hostname_bufsize);
-            valid = 1;
-        }
-
-        /* The port is out of range or the ip is an ipv6 or ipv4 address */
-        else;
-    }
-
-    /* No port found in the string, let's skip */
-    else;
-
-    return valid;
-}
-
 int
 main(int argc, char **argv)
 {
@@ -165,6 +100,14 @@ main(int argc, char **argv)
     */
     config_init(&options, argc, argv);
 
+#if HAVE_LIBFIU
+        /* Set any fault injection points early
+        */
+        if(! enable_fault_injections(&options))
+            clean_exit(ctx, &options, key, &key_len, hmac_key,
+                    &hmac_key_len, EXIT_FAILURE);
+#endif
+
     /* Handle previous execution arguments if required
     */
     if(prev_exec(&options, argc, argv) != 1)
@@ -257,12 +200,24 @@ main(int argc, char **argv)
         /* Resolve the client's public facing IP address if requestesd.
          * if this fails, consider it fatal.
         */
-        if (options.resolve_ip_http)
+        if (options.resolve_ip_http_https)
         {
-            if(resolve_ip_http(&options) < 0)
+            if(options.resolve_http_only)
             {
-                clean_exit(ctx, &options, key, &key_len,
-                    hmac_key, &hmac_key_len, EXIT_FAILURE);
+                if(resolve_ip_http(&options) < 0)
+                {
+                    clean_exit(ctx, &options, key, &key_len,
+                        hmac_key, &hmac_key_len, EXIT_FAILURE);
+                }
+            }
+            else
+            {
+                /* Default to HTTPS */
+                if(resolve_ip_https(&options) < 0)
+                {
+                    clean_exit(ctx, &options, key, &key_len,
+                        hmac_key, &hmac_key_len, EXIT_FAILURE);
+                }
             }
         }
 
@@ -329,6 +284,19 @@ main(int argc, char **argv)
                     hmac_key, &hmac_key_len, EXIT_FAILURE);
         }
 
+        /* Set gpg path if necessary
+        */
+        if(strlen(options.gpg_exe) > 0)
+        {
+            res = fko_set_gpg_exe(ctx, options.gpg_exe);
+            if(res != FKO_SUCCESS)
+            {
+                errmsg("fko_set_gpg_exe", res);
+                clean_exit(ctx, &options, key, &key_len,
+                        hmac_key, &hmac_key_len, EXIT_FAILURE);
+            }
+        }
+
         /* If a GPG home dir was specified, set it here.  Note: Setting
          * this has to occur before calling any of the other GPG-related
          * functions.
@@ -443,7 +411,8 @@ main(int argc, char **argv)
         if (res == FKO_SUCCESS)
             log_msg(LOG_VERBOSITY_NORMAL, "%s", dump_buf);
         else
-            log_msg(LOG_VERBOSITY_WARNING, "Unable to dump FKO context: %s", fko_errstr(res));
+            log_msg(LOG_VERBOSITY_WARNING, "Unable to dump FKO context: %s",
+                    fko_errstr(res));
     }
 
     /* Save packet data payload if requested.
@@ -451,6 +420,8 @@ main(int argc, char **argv)
     if (options.save_packet_file[0] != 0x0)
         write_spa_packet_data(ctx, &options);
 
+    /* SPA packet random destination port handling
+    */
     if (options.rand_port)
     {
         tmp_port = get_rand_port(ctx);
@@ -460,6 +431,22 @@ main(int argc, char **argv)
         options.spa_dst_port = tmp_port;
     }
 
+    /* If we are using one the "raw" modes (normally because
+     * we're going to spoof the SPA packet source IP), then select
+     * a random source port unless the source port is already set
+    */
+    if ((options.spa_proto == FKO_PROTO_TCP_RAW
+            || options.spa_proto == FKO_PROTO_UDP_RAW
+            || options.spa_proto == FKO_PROTO_ICMP)
+            && !options.spa_src_port)
+    {
+        tmp_port = get_rand_port(ctx);
+        if(tmp_port < 0)
+            clean_exit(ctx, &options, key, &orig_key_len,
+                    hmac_key, &hmac_key_len, EXIT_FAILURE);
+        options.spa_src_port = tmp_port;
+    }
+
     res = send_spa_packet(ctx, &options);
     if(res < 0)
     {
@@ -495,10 +482,10 @@ main(int argc, char **argv)
         if(res != FKO_SUCCESS)
         {
             errmsg("fko_get_spa_encryption_mode", res);
-            if(fko_destroy(ctx2) == FKO_ERROR_ZERO_OUT_DATA)
+            if(fko_destroy(ctx) == FKO_ERROR_ZERO_OUT_DATA)
                 log_msg(LOG_VERBOSITY_ERROR,
                         "[*] Could not zero out sensitive data buffer.");
-            ctx2 = NULL;
+            ctx = NULL;
             clean_exit(ctx, &options, key, &orig_key_len,
                 hmac_key, &hmac_key_len, EXIT_FAILURE);
         }
@@ -574,7 +561,10 @@ main(int argc, char **argv)
                  * expected, return 0 instead of an error condition (so calling
                  * programs like the fwknop test suite don't interpret this as
                  * an unrecoverable error), but print the error string for
-                 debugging purposes. */
+                 * debugging purposes. The test suite does run a series of
+                 * tests that use a single key pair for encryption and
+                 * authentication, so decryption become possible for these
+                 * tests. */
                 log_msg(LOG_VERBOSITY_ERROR, "GPG ERR: %s\n%s", fko_gpg_errstr(ctx2),
                     "No access to recipient private key?");
             }
@@ -585,12 +575,14 @@ main(int argc, char **argv)
             clean_exit(ctx, &options, key, &orig_key_len,
                 hmac_key, &hmac_key_len, EXIT_FAILURE);
         }
-
-        res = dump_ctx_to_buffer(ctx2, dump_buf, sizeof(dump_buf));
-        if (res == FKO_SUCCESS)
-            log_msg(LOG_VERBOSITY_NORMAL, "\nDump of the Decoded Data\n%s", dump_buf);
-        else
-            log_msg(LOG_VERBOSITY_WARNING, "Unable to dump FKO context: %s", fko_errstr(res));
+        /* Only dump out the SPA data after the test in verbose mode */
+        if (options.verbose) {
+            res = dump_ctx_to_buffer(ctx2, dump_buf, sizeof(dump_buf));
+            if (res == FKO_SUCCESS)
+                log_msg(LOG_VERBOSITY_NORMAL, "\nDump of the Decoded Data\n%s", dump_buf);
+            else
+                log_msg(LOG_VERBOSITY_WARNING, "Unable to dump FKO context: %s", fko_errstr(res));
+        }
 
         if(fko_destroy(ctx2) == FKO_ERROR_ZERO_OUT_DATA)
             log_msg(LOG_VERBOSITY_ERROR,
@@ -609,6 +601,8 @@ free_configs(fko_cli_options_t *opts)
 {
     if (opts->resolve_url != NULL)
         free(opts->resolve_url);
+    if (opts->wget_bin != NULL)
+        free(opts->wget_bin);
     zero_buf_wrapper(opts->key, MAX_KEY_LEN+1);
     zero_buf_wrapper(opts->key_base64, MAX_B64_KEY_LEN+1);
     zero_buf_wrapper(opts->hmac_key, MAX_KEY_LEN+1);
@@ -664,34 +658,6 @@ get_rand_port(fko_ctx_t ctx)
     return port;
 }
 
-/* See if the string is of the format "<ipv4 addr>:<port>",
- */
-static int
-ipv4_str_has_port(char *str)
-{
-    int o1, o2, o3, o4, p;
-
-    /* Force the ':' (if any) to a ','
-    */
-    char *ndx = strchr(str, ':');
-    if(ndx != NULL)
-        *ndx = ',';
-
-    /* Check format and values.
-    */
-    if((sscanf(str, "%u.%u.%u.%u,%u", &o1, &o2, &o3, &o4, &p)) == 5
-        && o1 >= 0 && o1 <= 255
-        && o2 >= 0 && o2 <= 255
-        && o3 >= 0 && o3 <= 255
-        && o4 >= 0 && o4 <= 255
-        && p  >  0 && p  <  65536)
-    {
-        return 1;
-    }
-
-    return 0;
-}
-
 /* Set access buf
 */
 static int
@@ -768,11 +734,9 @@ static int
 set_nat_access(fko_ctx_t ctx, fko_cli_options_t *options, const char * const access_buf)
 {
     char                nat_access_buf[MAX_LINE_LEN] = {0};
+    char                tmp_nat_port[MAX_LINE_LEN] = {0};
     char                tmp_access_port[MAX_PORT_STR_LEN+1] = {0}, *ndx = NULL;
-    int                 access_port = 0, i = 0, is_err = 0;
-    char                dst_ip_str[INET_ADDRSTRLEN] = {0};
-    char                hostname[HOSTNAME_BUFSIZE] = {0};
-    int                 port = 0;
+    int                 access_port = 0, i = 0, is_err = 0, hostlen = 0;
     struct addrinfo     hints;
 
     memset(&hints, 0 , sizeof(hints));
@@ -810,41 +774,65 @@ set_nat_access(fko_ctx_t ctx, fko_cli_options_t *options, const char * const acc
 
     if (nat_access_buf[0] == 0x0 && options->nat_access_str[0] != 0x0)
     {
-        if (ipv4_str_has_port(options->nat_access_str))
+        /* Force the ':' (if any) to a ','
+        */
+        ndx = strchr(options->nat_access_str, ':');
+        if (ndx != NULL)
+            *ndx = ',';
+
+        ndx = strchr(options->nat_access_str, ',');
+        if (ndx != NULL)
         {
-            snprintf(nat_access_buf, MAX_LINE_LEN, "%s",
-                options->nat_access_str);
+            hostlen = ndx - options->nat_access_str; //len of host, up til either comma or null
+            *ndx = 0;
+
+            ndx++;
+            i = 0;
+            while(*ndx != '\0')
+            //if it goes over max length, mark as invalid
+
+            {
+                tmp_nat_port[i] = *ndx;
+                if ((i > MAX_PORT_STR_LEN) || (!isdigit(*ndx)))
+                {
+                    log_msg(LOG_VERBOSITY_ERROR, "[*] Invalid port value in -N arg.");
+                    return FKO_ERROR_INVALID_DATA;
+                }
+                ndx++;
+                i++;
+            }
+            tmp_nat_port[i] = '\0';
+            access_port = strtol_wrapper(tmp_nat_port, 1,
+                        MAX_PORT, NO_EXIT_UPON_ERR, &is_err);
+            if (is_err != FKO_SUCCESS)
+            {
+                log_msg(LOG_VERBOSITY_ERROR, "[*] Invalid port value in -N arg.");
+                return FKO_ERROR_INVALID_DATA;
+            }
+        } else {
+            hostlen = strlen(options->nat_access_str);
         }
-        else
+
+        if ((access_port < 1) | (access_port > 65535))
+        {
+            log_msg(LOG_VERBOSITY_ERROR, "[*] Invalid port value.");
+            return FKO_ERROR_INVALID_DATA;
+        }
+
+
+        if (is_valid_ipv4_addr(options->nat_access_str, hostlen) || is_valid_hostname(options->nat_access_str, hostlen))
         {
             snprintf(nat_access_buf, MAX_LINE_LEN, NAT_ACCESS_STR_TEMPLATE,
                 options->nat_access_str, access_port);
         }
-    }
-
-    /* Check if there is a hostname to resolve as an ip address in the NAT access buffer */
-    if (is_hostname_str_with_port(nat_access_buf, hostname, sizeof(hostname), &port))
-    {
-        /* Speed up the name resolution by forcing ipv4 (AF_INET).
-         * A NULL pointer could be used instead if there is no constraint.
-         * Maybe when ipv6 support will be enable the structure could initialize the
-         * family to either AF_INET or AF_INET6 */
-        hints.ai_family = AF_INET;
-
-        if (resolve_dest_adr(hostname, &hints, dst_ip_str, sizeof(dst_ip_str)) != 0)
+        else
         {
-            log_msg(LOG_VERBOSITY_ERROR, "[*] Unable to resolve %s as an ip address",
-                    hostname);
+            log_msg(LOG_VERBOSITY_ERROR, "[*] Invalid NAT destination '%s' for -N arg.",
+                options->nat_access_str);
             return FKO_ERROR_INVALID_DATA;
         }
-
-        snprintf(nat_access_buf, MAX_LINE_LEN, NAT_ACCESS_STR_TEMPLATE,
-                dst_ip_str, port);
     }
 
-    /* Nothing to resolve */
-    else;
-
     if(options->nat_rand_port)
     {
         /* Must print to stdout what the random port is since
@@ -871,11 +859,20 @@ prev_exec(fko_cli_options_t *options, int argc, char **argv)
     }
     else
     {
-        if (get_save_file(args_save_file) != 1)
+        if(options->no_home_dir)
         {
-            log_msg(LOG_VERBOSITY_ERROR, "Unable to determine args save file");
+            log_msg(LOG_VERBOSITY_ERROR,
+                    "In --no-home-dir mode must set the args save file path with -E");
             return 0;
         }
+        else
+        {
+            if (get_save_file(args_save_file) != 1)
+            {
+                log_msg(LOG_VERBOSITY_ERROR, "Unable to determine args save file");
+                return 0;
+            }
+        }
     }
 
     if(options->run_last_command)
@@ -906,9 +903,13 @@ show_last_command(const char * const args_save_file)
     }
 
     if ((fgets(args_str, MAX_LINE_LEN, args_file_ptr)) != NULL) {
-        log_msg(LOG_VERBOSITY_NORMAL, "Last fwknop client command line: %s", args_str);
+        log_msg(LOG_VERBOSITY_NORMAL,
+                "Last fwknop client command line: %s", args_str);
     } else {
-        log_msg(LOG_VERBOSITY_NORMAL, "Could not read line from file: %s", args_save_file);
+        log_msg(LOG_VERBOSITY_NORMAL,
+                "Could not read line from file: %s", args_save_file);
+        fclose(args_file_ptr);
+        return 0;
     }
     fclose(args_file_ptr);
 
@@ -921,15 +922,12 @@ static int
 run_last_args(fko_cli_options_t *options, const char * const args_save_file)
 {
     FILE           *args_file_ptr = NULL;
-
-    int             current_arg_ctr = 0;
-    int             argc_new = 0;
-    int             i = 0;
-
-    char            args_str[MAX_LINE_LEN] = {0};
-    char            arg_tmp[MAX_LINE_LEN]  = {0};
+    int             argc_new = 0, args_broken = 0;
+    char            args_str[MAX_ARGS_LINE_LEN] = {0};
     char           *argv_new[MAX_CMDLINE_ARGS];  /* should be way more than enough */
 
+    memset(argv_new, 0x0, sizeof(argv_new));
+
     if(verify_file_perms_ownership(args_save_file) != 1)
         return 0;
 
@@ -944,37 +942,16 @@ run_last_args(fko_cli_options_t *options, const char * const args_save_file)
         args_str[MAX_LINE_LEN-1] = '\0';
         if (options->verbose)
             log_msg(LOG_VERBOSITY_NORMAL, "Executing: %s", args_str);
-        for (i=0; i < (int)strlen(args_str); i++)
+        if(strtoargv(args_str, argv_new, &argc_new) != 1)
         {
-            if (!isspace(args_str[i]))
-            {
-                arg_tmp[current_arg_ctr] = args_str[i];
-                current_arg_ctr++;
-            }
-            else
-            {
-                arg_tmp[current_arg_ctr] = '\0';
-                argv_new[argc_new] = malloc(strlen(arg_tmp)+1);
-                if (argv_new[argc_new] == NULL)
-                {
-                    log_msg(LOG_VERBOSITY_ERROR, "[*] malloc failure for cmd line arg.");
-                    fclose(args_file_ptr);
-                    return 0;
-                }
-                strlcpy(argv_new[argc_new], arg_tmp, strlen(arg_tmp)+1);
-                current_arg_ctr = 0;
-                argc_new++;
-                if(argc_new >= MAX_CMDLINE_ARGS)
-                {
-                    log_msg(LOG_VERBOSITY_ERROR, "[*] max command line args exceeded.");
-                    fclose(args_file_ptr);
-                    return 0;
-                }
-            }
+            args_broken = 1;
         }
     }
     fclose(args_file_ptr);
 
+    if(args_broken)
+        return 0;
+
     /* Reset the options index so we can run through them again.
     */
     optind = 0;
@@ -983,13 +960,7 @@ run_last_args(fko_cli_options_t *options, const char * const args_save_file)
 
     /* Since we passed in our own copies, free up malloc'd memory
     */
-    for (i=0; i < argc_new; i++)
-    {
-        if(argv_new[i] == NULL)
-            break;
-        else
-            free(argv_new[i]);
-    }
+    free_argv(argv_new, &argc_new);
 
     return 1;
 }
@@ -1092,7 +1063,9 @@ static int
 get_keys(fko_ctx_t ctx, fko_cli_options_t *options,
     char *key, int *key_len, char *hmac_key, int *hmac_key_len)
 {
+#if !AFL_FUZZING
     char   *key_tmp = NULL, *hmac_key_tmp = NULL;
+#endif
     int     use_hmac = 0, res = 0;
 
     memset(key, 0x0, MAX_KEY_LEN+1);
@@ -1139,6 +1112,9 @@ get_keys(fko_ctx_t ctx, fko_cli_options_t *options,
                     "[+] GPG mode set, signing passphrase not required");
             else if(strlen(options->gpg_signer_key))
             {
+#if AFL_FUZZING
+                strlcpy(key, AFL_ENC_KEY, MAX_KEY_LEN+1);
+#else
                 key_tmp = getpasswd("Enter passphrase for signing: ", options->input_fd);
                 if(key_tmp == NULL)
                 {
@@ -1146,19 +1122,23 @@ get_keys(fko_ctx_t ctx, fko_cli_options_t *options,
                     return 0;
                 }
                 strlcpy(key, key_tmp, MAX_KEY_LEN+1);
+#endif
                 *key_len = strlen(key);
             }
         }
         else
         {
+#if AFL_FUZZING
+            strlcpy(key, AFL_ENC_KEY, MAX_KEY_LEN+1);
+#else
             key_tmp = getpasswd("Enter encryption key: ", options->input_fd);
-
             if(key_tmp == NULL)
             {
                 log_msg(LOG_VERBOSITY_ERROR, "[*] getpasswd() key error.");
                 return 0;
             }
             strlcpy(key, key_tmp, MAX_KEY_LEN+1);
+#endif
             *key_len = strlen(key);
         }
     }
@@ -1198,15 +1178,17 @@ get_keys(fko_ctx_t ctx, fko_cli_options_t *options,
         }
         else
         {
+#if AFL_FUZZING
+            strlcpy(hmac_key, AFL_HMAC_KEY, MAX_KEY_LEN+1);
+#else
             hmac_key_tmp = getpasswd("Enter HMAC key: ", options->input_fd);
-
             if(hmac_key_tmp == NULL)
             {
                 log_msg(LOG_VERBOSITY_ERROR, "[*] getpasswd() key error.");
                 return 0;
             }
-
             strlcpy(hmac_key, hmac_key_tmp, MAX_KEY_LEN+1);
+#endif
             *hmac_key_len = strlen(hmac_key);
             use_hmac = 1;
         }
@@ -1266,6 +1248,33 @@ zero_buf_wrapper(char *buf, int len)
     return;
 }
 
+#if HAVE_LIBFIU
+static int
+enable_fault_injections(fko_cli_options_t * const opts)
+{
+    int rv = 1;
+    if(opts->fault_injection_tag[0] != 0x0)
+    {
+        if(opts->verbose)
+            log_msg(LOG_VERBOSITY_NORMAL, "[+] Enable fault injection tag: %s",
+                    opts->fault_injection_tag);
+        if(fiu_init(0) != 0)
+        {
+            log_msg(LOG_VERBOSITY_WARNING, "[*] Unable to set fault injection tag: %s",
+                    opts->fault_injection_tag);
+            rv = 0;
+        }
+        if(fiu_enable(opts->fault_injection_tag, 1, NULL, 0) != 0)
+        {
+            log_msg(LOG_VERBOSITY_WARNING, "[*] Unable to set fault injection tag: %s",
+                    opts->fault_injection_tag);
+            rv = 0;
+        }
+    }
+    return rv;
+}
+#endif
+
 /* free up memory and exit
 */
 static void
@@ -1273,6 +1282,11 @@ clean_exit(fko_ctx_t ctx, fko_cli_options_t *opts,
         char *key, int *key_len, char *hmac_key, int *hmac_key_len,
         unsigned int exit_status)
 {
+#if HAVE_LIBFIU
+    if(opts->fault_injection_tag[0] != 0x0)
+        fiu_disable(opts->fault_injection_tag);
+#endif
+
     if(fko_destroy(ctx) == FKO_ERROR_ZERO_OUT_DATA)
         log_msg(LOG_VERBOSITY_ERROR,
                 "[*] Could not zero out sensitive data buffer.");
