@@ -1,14 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file client/spa_comm.c
  *
- * File:    spa_comm.c
- *
- * Author:  Damien S. Stuart (dstuart@dstuart.org)
- *          Michael Rash (mbr@cipherdyne.org)
- *
- * Purpose: Network-related functions for the fwknop client
- *
- * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Network-related functions for the fwknop client
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -55,7 +53,7 @@ dump_transmit_options(const fko_cli_options_t *options)
 
 /* Function to generate a header checksum.
 */
-unsigned short
+static unsigned short
 chksum(unsigned short *buf, int nbytes)
 {
     unsigned int   sum;
@@ -87,8 +85,8 @@ static int
 send_spa_packet_tcp_or_udp(const char *spa_data, const int sd_len,
     const fko_cli_options_t *options)
 {
-    int     sock, res=0, error;
-    struct  addrinfo *result, *rp, hints;
+    int     sock=-1, sock_success=0, res=0, error;
+    struct  addrinfo *result=NULL, *rp, hints;
     char    port_str[MAX_PORT_STR_LEN+1] = {0};
 
     if (options->test)
@@ -100,7 +98,7 @@ send_spa_packet_tcp_or_udp(const char *spa_data, const int sd_len,
 
     memset(&hints, 0, sizeof(struct addrinfo));
 
-    hints.ai_family   = AF_UNSPEC; /* Allow IPv4 or IPv6 */
+    hints.ai_family   = AF_INET; /* Allow IPv4 only */
 
     if (options->spa_proto == FKO_PROTO_UDP)
     {
@@ -120,6 +118,14 @@ send_spa_packet_tcp_or_udp(const char *spa_data, const int sd_len,
 
     snprintf(port_str, MAX_PORT_STR_LEN+1, "%d", options->spa_dst_port);
 
+#if AFL_FUZZING
+    /* Make sure to never send SPA packets under AFL fuzzing cycles
+    */
+    log_msg(LOG_VERBOSITY_NORMAL,
+        "AFL fuzzing enabled, SPA packet not actually sent.");
+    return res;
+#endif
+
     error = getaddrinfo(options->spa_server_str, port_str, &hints, &result);
 
     if (error != 0)
@@ -129,30 +135,46 @@ send_spa_packet_tcp_or_udp(const char *spa_data, const int sd_len,
     }
 
     for (rp = result; rp != NULL; rp = rp->ai_next) {
+        /* Apply --server-resolve-ipv4 criteria
+        */
+        if(options->spa_server_resolve_ipv4)
+        {
+            if(rp->ai_family != AF_INET)
+            {
+                log_msg(LOG_VERBOSITY_DEBUG, "Non-IPv4 resolution");
+                continue;
+            }
+        }
+
         sock = socket(rp->ai_family, rp->ai_socktype,
                 rp->ai_protocol);
         if (sock < 0)
             continue;
 
         if ((error = (connect(sock, rp->ai_addr, rp->ai_addrlen) != -1)))
+        {
+            sock_success = 1;
             break;  /* made it */
-
+        }
+        else /* close the open socket if there was a connect error */
+        {
 #ifdef WIN32
-        closesocket(sock);
+            closesocket(sock);
 #else
-        close(sock);
+            close(sock);
 #endif
+        }
     }
+    if(result != NULL)
+        freeaddrinfo(result);
 
-    if (rp == NULL) {
+    if (! sock_success) {
         log_msg(LOG_VERBOSITY_ERROR,
                 "send_spa_packet_tcp_or_udp: Could not create socket: ",
                 strerror(errno));
         return -1;
     }
 
-    freeaddrinfo(result);
-
     res = send(sock, spa_data, sd_len, 0);
 
     if(res < 0)
@@ -257,7 +279,7 @@ send_spa_packet_tcp_raw(const char *spa_data, const int sd_len,
     tcph->check     = 0;
     tcph->urg_ptr   = 0;
 
-    /* No we can compute our checksum.
+    /* Now we can compute our checksum.
     */
     iph->check = chksum((unsigned short *)pkt_data, iph->tot_len);
 
@@ -354,9 +376,9 @@ send_spa_packet_udp_raw(const char *spa_data, const int sd_len,
     udph->source    = saddr->sin_port;
     udph->dest      = daddr->sin_port;
     udph->check     = 0;
-    udph->len       = sd_len + sizeof(struct udphdr);
+    udph->len       = htons(sd_len + sizeof(struct udphdr));
 
-    /* No we can compute our checksum.
+    /* Now we can compute our checksum.
     */
     iph->check = chksum((unsigned short *)pkt_data, iph->tot_len);
 
@@ -460,7 +482,7 @@ send_spa_packet_icmp(const char *spa_data, const int sd_len,
         icmph->un.echo.sequence = htons(1);
     }
 
-    /* No we can compute our checksum.
+    /* Now we can compute our checksum.
     */
     iph->check = chksum((unsigned short *)pkt_data, iph->tot_len);
     icmph->checksum = chksum((unsigned short *)icmph, sizeof(struct icmphdr) + sd_len);
@@ -534,7 +556,7 @@ send_spa_packet_http(const char *spa_data, const int sd_len,
     else /* we are sending the SPA packet through an HTTP proxy */
     {
         /* Extract the hostname if it was specified as a URL. Actually,
-         * we just move the start of the hostname to the begining of the
+         * we just move the start of the hostname to the beginning of the
          * original string.
         */
         if(strncasecmp(ndx, "http://", 7) == 0)
@@ -586,6 +608,9 @@ send_spa_packet_http(const char *spa_data, const int sd_len,
         return 0;
     }
 
+    /* In AFL fuzzing mode, the following function will not send
+     * the SPA packet.
+    */
     return send_spa_packet_tcp_or_udp(http_buf, strlen(http_buf), options);
 }
 
@@ -597,8 +622,8 @@ send_spa_packet(fko_ctx_t ctx, fko_cli_options_t *options)
     int                 res, sd_len;
     char               *spa_data;
     struct sockaddr_in  saddr, daddr;
-    char                ip_str[INET_ADDRSTRLEN] = {0};  /* String used to contain the ip addres of an hostname */
-    struct addrinfo     hints;                          /* Structure used to set hints to reslove hostname */
+    char                ip_str[INET_ADDRSTRLEN] = {0};  /* String used to contain the ip address of an hostname */
+    struct addrinfo     hints;                          /* Structure used to set hints to resolve hostname */
 #ifdef WIN32
     WSADATA wsa_data;
 #endif
@@ -659,7 +684,7 @@ send_spa_packet(fko_ctx_t ctx, fko_cli_options_t *options)
         if (options->spa_src_port)
             saddr.sin_port = htons(options->spa_src_port);
         else
-            saddr.sin_port = INADDR_ANY;  /* default */
+            saddr.sin_port = INADDR_ANY;
 
         if (options->spoof_ip_src_str[0] != 0x00) {
             saddr.sin_addr.s_addr = inet_addr(options->spoof_ip_src_str);
@@ -676,11 +701,20 @@ send_spa_packet(fko_ctx_t ctx, fko_cli_options_t *options)
         */
         daddr.sin_port = htons(options->spa_dst_port);
 
-        /* Set destination address. We use the default protocol to reslove
+        /* Set destination address. We use the default protocol to resolve
          * the ip address */
         hints.ai_family = AF_INET;
 
-        if (resolve_dest_adr(options->spa_server_str, &hints, ip_str, sizeof(ip_str)) != 0)
+#if AFL_FUZZING
+        /* Make sure to never send SPA packets under AFL fuzzing cycles
+        */
+        log_msg(LOG_VERBOSITY_NORMAL,
+            "AFL fuzzing enabled, SPA packet not actually sent.");
+        return res;
+#endif
+
+        if (resolve_dst_addr(options->spa_server_str,
+                    &hints, ip_str, sizeof(ip_str), options) != 0)
         {
             log_msg(LOG_VERBOSITY_ERROR, "[*] Unable to resolve %s as an ip address",
                     options->spa_server_str);
