@@ -1,13 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file lib/fko_common.h
  *
- * File:    fko_common.h
- *
- * Author:  Damien S. Stuart
- *
- * Purpose: Common header for libfko source files.
- *
- * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Common header for libfko source files.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -35,6 +34,10 @@
   #include "config.h"
 #endif
 
+#if HAVE_LIBFIU
+  #include <fiu.h>
+#endif
+
 #include <stdio.h>
 #include <sys/types.h>
 
@@ -69,7 +72,7 @@
   #define O_WRONLY    _O_WRONLY
   #define O_RDONLY    _O_RDONLY
   #define O_RDWR      _O_RDWR
-  #define O_CREAT     _O_CREAT 
+  #define O_CREAT     _O_CREAT
   #define O_EXCL      _O_EXCL
   #define S_IRUSR     _S_IREAD
   #define S_IWUSR     _S_IWRITE
@@ -86,7 +89,7 @@
   #endif
 #endif
 
-/* Work out endianess (sp?)
+/* Work out endianness
 */
 #if HAVE_ENDIAN_H       /* Should cover most Linux systems */
   #include <endian.h>
