@@ -1,13 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file lib/fko_encryption.c
  *
- * File:    fko_encryption.c
- *
- * Author:  Damien S. Stuart
- *
- * Purpose: Set/Get the spa encryption type.
- *
- * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Set/Get the spa encryption type.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -114,7 +113,7 @@ _rijndael_encrypt(fko_ctx_t ctx, const char *enc_key, const int enc_key_len)
 
     /* Now make a bucket for the base64-encoded version and populate it.
     */
-    b64ciphertext = malloc(((cipher_len / 3) * 4) + 8);
+    b64ciphertext = calloc(1, ((cipher_len / 3) * 4) + 8);
     if(b64ciphertext == NULL)
     {
         if(zero_free((char *) ciphertext, pt_len+32) == FKO_SUCCESS
@@ -131,8 +130,7 @@ _rijndael_encrypt(fko_ctx_t ctx, const char *enc_key, const int enc_key_len)
         zero_free_rv = zero_free(ctx->encrypted_msg,
                 strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE));
 
-    ctx->encrypted_msg     = strdup(b64ciphertext);
-    ctx->encrypted_msg_len = strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE);
+    ctx->encrypted_msg = strdup(b64ciphertext);
 
     /* Clean-up
     */
@@ -149,6 +147,8 @@ _rijndael_encrypt(fko_ctx_t ctx, const char *enc_key, const int enc_key_len)
     if(ctx->encrypted_msg == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
+    ctx->encrypted_msg_len = strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE);
+
     if(! is_valid_encoded_msg_len(ctx->encrypted_msg_len))
         return(FKO_ERROR_INVALID_DATA_ENCRYPT_RESULT_MSGLEN_VALIDFAIL);
 
@@ -163,7 +163,7 @@ _rijndael_decrypt(fko_ctx_t ctx,
 {
     unsigned char  *ndx;
     unsigned char  *cipher;
-    int             cipher_len, pt_len, i, err = 0, res = FKO_SUCCESS;
+    int             cipher_len=0, pt_len, i, err = 0, res = FKO_SUCCESS;
     int             zero_free_rv = FKO_SUCCESS;
 
     if(key_len < 0 || key_len > RIJNDAEL_MAX_KEYSIZE)
@@ -182,10 +182,14 @@ _rijndael_decrypt(fko_ctx_t ctx,
     /* Create a bucket for the (base64) decoded encrypted data and get the
      * raw cipher data.
     */
-    cipher = malloc(ctx->encrypted_msg_len);
+    cipher = calloc(1, ctx->encrypted_msg_len);
     if(cipher == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
+#if AFL_FUZZING
+    cipher_len = ctx->encrypted_msg_len;
+    memcpy(cipher, ctx->encrypted_msg, ctx->encrypted_msg_len);
+#else
     if((cipher_len = b64_decode(ctx->encrypted_msg, cipher)) < 0)
     {
         if(zero_free((char *)cipher, ctx->encrypted_msg_len) == FKO_SUCCESS)
@@ -193,6 +197,7 @@ _rijndael_decrypt(fko_ctx_t ctx,
         else
             return(FKO_ERROR_ZERO_OUT_DATA);
     }
+#endif
 
     /* Since we're using AES, make sure the incoming data is a multiple of
      * the blocksize
@@ -212,7 +217,7 @@ _rijndael_decrypt(fko_ctx_t ctx,
     /* Create a bucket for the plaintext data and decrypt the message
      * data into it.
     */
-    ctx->encoded_msg = malloc(cipher_len);
+    ctx->encoded_msg = calloc(1, cipher_len);
     if(ctx->encoded_msg == NULL)
     {
         if(zero_free((char *)cipher, ctx->encrypted_msg_len) == FKO_SUCCESS)
@@ -236,10 +241,10 @@ _rijndael_decrypt(fko_ctx_t ctx,
         return(FKO_ERROR_DECRYPTION_SIZE);
 
     if(ctx->encoded_msg == NULL)
-        return(FKO_ERROR_INVALID_DATA_ENCRYPT_GPG_ENCODEDMSG_NULL);
+        return(FKO_ERROR_MISSING_ENCODED_DATA);
 
     if(! is_valid_encoded_msg_len(pt_len))
-        return(FKO_ERROR_INVALID_DATA_ENCRYPT_GPG_ENCODEDMSGLEN_VALIDFAIL);
+        return(FKO_ERROR_INVALID_DATA_DECODE_MSGLEN_VALIDFAIL);
 
     if(zero_free_rv != FKO_SUCCESS)
         return(zero_free_rv);
@@ -309,7 +314,7 @@ gpg_encrypt(fko_ctx_t ctx, const char *enc_key)
     /* Make a bucket big enough to hold the enc msg + digest (plaintext)
      * and populate it appropriately.
     */
-    plain = malloc(ctx->encoded_msg_len + ctx->digest_len + 2);
+    plain = calloc(1, ctx->encoded_msg_len + ctx->digest_len + 2);
     if(plain == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
@@ -354,7 +359,7 @@ gpg_encrypt(fko_ctx_t ctx, const char *enc_key)
 
     /* Now make a bucket for the base64-encoded version and populate it.
     */
-    b64cipher = malloc(((cipher_len / 3) * 4) + 8);
+    b64cipher = calloc(1, ((cipher_len / 3) * 4) + 8);
     if(b64cipher == NULL)
     {
         if(zero_free(plain, pt_len) != FKO_SUCCESS)
@@ -377,8 +382,7 @@ gpg_encrypt(fko_ctx_t ctx, const char *enc_key)
         zero_free_rv = zero_free(ctx->encrypted_msg,
                 strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE));
 
-    ctx->encrypted_msg     = strdup(b64cipher);
-    ctx->encrypted_msg_len = strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE);
+    ctx->encrypted_msg = strdup(b64cipher);
 
     /* Clean-up
     */
@@ -395,6 +399,8 @@ gpg_encrypt(fko_ctx_t ctx, const char *enc_key)
     if(ctx->encrypted_msg == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
+    ctx->encrypted_msg_len = strnlen(ctx->encrypted_msg, MAX_SPA_ENCODED_MSG_SIZE);
+
     if(! is_valid_encoded_msg_len(ctx->encrypted_msg_len))
         return(FKO_ERROR_INVALID_DATA_ENCRYPT_GPG_RESULT_MSGLEN_VALIDFAIL);
 
@@ -419,7 +425,7 @@ gpg_decrypt(fko_ctx_t ctx, const char *dec_key)
     /* Create a bucket for the (base64) decoded encrypted data and get the
      * raw cipher data.
     */
-    cipher = malloc(ctx->encrypted_msg_len);
+    cipher = calloc(1, ctx->encrypted_msg_len);
     if(cipher == NULL)
         return(FKO_ERROR_MEMORY_ALLOCATION);
 
@@ -450,13 +456,11 @@ gpg_decrypt(fko_ctx_t ctx, const char *dec_key)
 
     /* Done with cipher...
     */
-    if(res != FKO_SUCCESS)
-    {
-        if(zero_free((char *) cipher, ctx->encrypted_msg_len) != FKO_SUCCESS)
-            return(FKO_ERROR_ZERO_OUT_DATA);
-        else
+    if(zero_free((char *) cipher, ctx->encrypted_msg_len) != FKO_SUCCESS)
+        return(FKO_ERROR_ZERO_OUT_DATA);
+    else
+        if(res != FKO_SUCCESS) /* bail if there was some other problem */
             return(res);
-    }
 
     pt_len = strnlen(ctx->encoded_msg, MAX_SPA_ENCODED_MSG_SIZE);
 
@@ -480,11 +484,19 @@ gpg_decrypt(fko_ctx_t ctx, const char *dec_key)
 int
 fko_set_spa_encryption_type(fko_ctx_t ctx, const short encrypt_type)
 {
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_spa_encryption_type_init",
+            FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
         return(FKO_ERROR_CTX_NOT_INITIALIZED);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_spa_encryption_type_val",
+            FKO_ERROR_INVALID_DATA_ENCRYPT_TYPE_VALIDFAIL);
+#endif
     if(encrypt_type < 0 || encrypt_type >= FKO_LAST_ENCRYPTION_TYPE)
         return(FKO_ERROR_INVALID_DATA_ENCRYPT_TYPE_VALIDFAIL);
 
@@ -515,11 +527,19 @@ fko_get_spa_encryption_type(fko_ctx_t ctx, short *enc_type)
 int
 fko_set_spa_encryption_mode(fko_ctx_t ctx, const int encrypt_mode)
 {
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_spa_encryption_mode_init",
+            FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
         return(FKO_ERROR_CTX_NOT_INITIALIZED);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_spa_encryption_mode_val",
+            FKO_ERROR_INVALID_DATA_ENCRYPT_MODE_VALIDFAIL);
+#endif
     if(encrypt_mode < 0 || encrypt_mode >= FKO_LAST_ENC_MODE)
         return(FKO_ERROR_INVALID_DATA_ENCRYPT_MODE_VALIDFAIL);
 
@@ -1104,6 +1124,7 @@ fko_gpg_signature_id_match(fko_ctx_t ctx, const char * const id,
 {
 #if HAVE_LIBGPGME
     char *curr_id;
+    int   rv = FKO_SUCCESS;
 
     /* Must be initialized
     */
@@ -1125,7 +1146,9 @@ fko_gpg_signature_id_match(fko_ctx_t ctx, const char * const id,
     if(ctx->gpg_sigs == NULL)
         return(FKO_ERROR_GPGME_NO_SIGNATURE);
 
-    fko_get_gpg_signature_id(ctx, &curr_id);
+    rv = fko_get_gpg_signature_id(ctx, &curr_id);
+    if(rv != FKO_SUCCESS)
+        return rv;
 
     *result = strcmp(id, curr_id) == 0 ? 1 : 0;
 
