@@ -1,13 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file lib/fko_error.c
  *
- * File:    fko_error.c
- *
- * Author:  Damien S. Stuart
- *
- * Purpose: Error handling functions for libfko
- *
- * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Error handling functions for libfko
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -69,6 +68,12 @@ fko_errstr(const int err_code)
         case FKO_ERROR_INVALID_DATA_DECODE_LT_MIN_FIELDS:
             return("Args contain invalid data: FKO_ERROR_INVALID_DATA_DECODE_LT_MIN_FIELDS");
 
+        case FKO_ERROR_INVALID_DATA_DECODE_GT_MAX_FIELDS:
+            return("Args contain invalid data: FKO_ERROR_INVALID_DATA_DECODE_GT_MAX_FIELDS");
+
+        case FKO_ERROR_INVALID_DATA_DECODE_WRONG_NUM_FIELDS:
+            return("Args contain invalid data: FKO_ERROR_INVALID_DATA_DECODE_WRONG_NUM_FIELDS");
+
         case FKO_ERROR_INVALID_DATA_DECODE_ENC_MSG_LEN_MT_T_SIZE:
             return("Args contain invalid data: FKO_ERROR_INVALID_DATA_DECODE_ENC_MSG_LEN_MT_T_SIZE");
 
@@ -315,8 +320,8 @@ fko_errstr(const int err_code)
         case FKO_ERROR_INVALID_DATA_UTIL_STRTOL_LT_MIN:
             return("Args contain invalid data: FKO_ERROR_INVALID_DATA_UTIL_STRTOL_LT_MIN");
 
-        case FKO_ERROR_INVALID_DATA_UTIL_STROL_GT_MAX:
-            return("Args contain invalid data: FKO_ERROR_INVALID_DATA_UTIL_STROL_GT_MAX");
+        case FKO_ERROR_INVALID_DATA_UTIL_STRTOL_GT_MAX:
+            return("Args contain invalid data: FKO_ERROR_INVALID_DATA_UTIL_STRTOL_GT_MAX");
 
         case FKO_ERROR_DATA_TOO_LARGE:
             return("Value or Size of the data exceeded the max allowed");
