@@ -1,14 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file lib/fko_message.c
  *
- * File:    fko_message.c
- *
- * Author:  Damien S. Stuart
- *
- * Purpose: Set/Get the spa message (access req/command/etc) based
- *          on the current spa data.
- *
- * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Set/Get the spa message (access req/command/etc) based on the current spa data.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -66,7 +64,7 @@ have_allow_ip(const char *msg)
         res = FKO_ERROR_INVALID_ALLOW_IP;
 
     if(res == FKO_SUCCESS)
-        if (! is_valid_ipv4_addr(ip_str))
+        if (! is_valid_ipv4_addr(ip_str, strlen(ip_str)))
             res = FKO_ERROR_INVALID_ALLOW_IP;
 
     return(res);
@@ -75,8 +73,10 @@ have_allow_ip(const char *msg)
 static int
 have_port(const char *msg)
 {
-    const char         *ndx  = msg;
-    int     startlen         = strnlen(msg, MAX_SPA_MESSAGE_SIZE), port_str_len = 0;
+    const char  *ndx = msg;
+    char        port_str[MAX_PORT_STR_LEN+1] = {0};
+    int         startlen = strnlen(msg, MAX_SPA_MESSAGE_SIZE);
+    int         port_str_len=0, i=0, is_err;
 
     if(startlen == MAX_SPA_MESSAGE_SIZE)
         return(FKO_ERROR_INVALID_DATA_MESSAGE_PORT_MISSING);
@@ -91,8 +91,15 @@ have_port(const char *msg)
         port_str_len++;
         if((isdigit(*ndx) == 0) || (port_str_len > MAX_PORT_STR_LEN))
             return(FKO_ERROR_INVALID_SPA_ACCESS_MSG);
+        port_str[i] = *ndx;
         ndx++;
+        i++;
     }
+    port_str[i] = '\0';
+
+    strtol_wrapper(port_str, 1, MAX_PORT, NO_EXIT_UPON_ERR, &is_err);
+    if(is_err != FKO_SUCCESS)
+        return(FKO_ERROR_INVALID_SPA_ACCESS_MSG);
 
     return FKO_SUCCESS;
 }
@@ -102,11 +109,19 @@ have_port(const char *msg)
 int
 fko_set_spa_message_type(fko_ctx_t ctx, const short msg_type)
 {
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_spa_message_type_init",
+            FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
         return FKO_ERROR_CTX_NOT_INITIALIZED;
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_spa_message_type_val",
+            FKO_ERROR_INVALID_DATA_MESSAGE_TYPE_VALIDFAIL);
+#endif
     if(msg_type < 0 || msg_type >= FKO_LAST_MSG_TYPE)
         return(FKO_ERROR_INVALID_DATA_MESSAGE_TYPE_VALIDFAIL);
 
@@ -122,6 +137,12 @@ fko_set_spa_message_type(fko_ctx_t ctx, const short msg_type)
 int
 fko_get_spa_message_type(fko_ctx_t ctx, short *msg_type)
 {
+
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_spa_message_type_init",
+            FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
+
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
@@ -130,6 +151,10 @@ fko_get_spa_message_type(fko_ctx_t ctx, short *msg_type)
     if(msg_type == NULL)
         return(FKO_ERROR_INVALID_DATA);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_spa_message_type_val", FKO_ERROR_INVALID_DATA);
+#endif
+
     *msg_type = ctx->message_type;
 
     return(FKO_SUCCESS);
@@ -168,7 +193,7 @@ fko_set_spa_message(fko_ctx_t ctx, const char * const msg)
     if(res != FKO_SUCCESS)
         return(res);
 
-    /* Just in case this is a subsquent call to this function.  We
+    /* Just in case this is a subsequent call to this function.  We
      * do not want to be leaking memory.
     */
     if(ctx->message != NULL)
@@ -189,6 +214,11 @@ fko_set_spa_message(fko_ctx_t ctx, const char * const msg)
 int
 fko_get_spa_message(fko_ctx_t ctx, char **msg)
 {
+
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_spa_message_init", FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
+
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
@@ -197,6 +227,10 @@ fko_get_spa_message(fko_ctx_t ctx, char **msg)
     if(msg == NULL)
         return(FKO_ERROR_INVALID_DATA);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_spa_message_val", FKO_ERROR_INVALID_DATA);
+#endif
+
     *msg = ctx->message;
 
     return(FKO_SUCCESS);
@@ -268,15 +302,27 @@ int
 validate_nat_access_msg(const char *msg)
 {
     const char   *ndx;
+    int     host_len;
     int     res         = FKO_SUCCESS;
     int     startlen    = strnlen(msg, MAX_SPA_MESSAGE_SIZE);
 
     if(startlen == MAX_SPA_MESSAGE_SIZE)
         return(FKO_ERROR_INVALID_DATA_MESSAGE_NAT_MISSING);
 
-    /* Should always have a valid allow IP regardless of message type
+    /* must have exactly one comma here
     */
-    if((res = have_allow_ip(msg)) != FKO_SUCCESS)
+    if(count_characters(msg, ',', startlen) != 1)
+        return(FKO_ERROR_INVALID_SPA_NAT_ACCESS_MSG);
+
+    /* Must not be longer than the max hostname length
+    */
+    host_len = strcspn(msg, ",");
+    if(host_len > MAX_HOSTNAME_LEN)
+        return(FKO_ERROR_INVALID_SPA_NAT_ACCESS_MSG);
+
+    /* Check for some invalid characters
+    */
+    if(strcspn(msg, " /?\"\'\\") < host_len)
         return(FKO_ERROR_INVALID_SPA_NAT_ACCESS_MSG);
 
     /* Position ourselves beyond the allow IP and make sure we have
