@@ -1,13 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file lib/fko_server_auth.c
  *
- * File:    fko_server_auth.c
- *
- * Author:  Damien S. Stuart
- *
- * Purpose: Set/Get the spa server auth data.
- *
- * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Set/Get the spa server auth data.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -42,6 +41,9 @@ fko_set_spa_server_auth(fko_ctx_t ctx, const char * const msg)
     */
     //return(FKO_ERROR_UNSUPPORTED_FEATURE);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_spa_server_auth_init", FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
 
     /* Context must be initialized.
     */
@@ -66,7 +68,7 @@ fko_set_spa_server_auth(fko_ctx_t ctx, const char * const msg)
 
     /**/
 
-    /* Just in case this is a subsquent call to this function.  We
+    /* Just in case this is a subsequent call to this function.  We
      * do not want to be leaking memory.
     */
     if(ctx->server_auth != NULL)
@@ -87,6 +89,11 @@ fko_set_spa_server_auth(fko_ctx_t ctx, const char * const msg)
 int
 fko_get_spa_server_auth(fko_ctx_t ctx, char **server_auth)
 {
+
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_spa_server_auth_init", FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
+
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
@@ -95,6 +102,10 @@ fko_get_spa_server_auth(fko_ctx_t ctx, char **server_auth)
     if(server_auth == NULL)
         return(FKO_ERROR_INVALID_DATA);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_spa_server_auth_val", FKO_ERROR_INVALID_DATA);
+#endif
+
     *server_auth = ctx->server_auth;
 
     return(FKO_SUCCESS);
