@@ -1,13 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file lib/fko_timestamp.c
  *
- * File:    fko_timestamp.c
- *
- * Author:  Damien S. Stuart
- *
- * Purpose: Get the current timestamp with optional offset applied.
- *
- * Copyright 2009-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Get the current timestamp with optional offset applied.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -39,6 +38,10 @@ fko_set_timestamp(fko_ctx_t ctx, const int offset)
 {
     time_t ts;
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_timestamp_init", FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
+
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
@@ -46,6 +49,10 @@ fko_set_timestamp(fko_ctx_t ctx, const int offset)
 
     ts = time(NULL) + offset;
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_set_timestamp_val",
+            FKO_ERROR_INVALID_DATA_TIMESTAMP_VALIDFAIL);
+#endif
     if(ts < 0)
         return(FKO_ERROR_INVALID_DATA_TIMESTAMP_VALIDFAIL);
 
@@ -61,6 +68,11 @@ fko_set_timestamp(fko_ctx_t ctx, const int offset)
 int
 fko_get_timestamp(fko_ctx_t ctx, time_t *timestamp)
 {
+
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_timestamp_init", FKO_ERROR_CTX_NOT_INITIALIZED);
+#endif
+
     /* Must be initialized
     */
     if(!CTX_INITIALIZED(ctx))
@@ -69,6 +81,10 @@ fko_get_timestamp(fko_ctx_t ctx, time_t *timestamp)
     if(timestamp == NULL)
         return(FKO_ERROR_INVALID_DATA);
 
+#if HAVE_LIBFIU
+    fiu_return_on("fko_get_timestamp_val", FKO_ERROR_INVALID_DATA);
+#endif
+
     *timestamp = ctx->timestamp;
 
     return(FKO_SUCCESS);
