@@ -1,13 +1,12 @@
-/*
- ******************************************************************************
+/**
+ * \file server/config_init.c
  *
- * File:    config_init.c
- *
- * Author:  Damien Stuart
- *
- * Purpose: Command-line and config file processing for fwknop server.
- *
- * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Command-line and config file processing for fwknop server.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -29,25 +28,28 @@
  ******************************************************************************
 */
 #include "fwknopd_common.h"
+#include "fwknopd_errors.h"
 #include "config_init.h"
 #include "access.h"
 #include "cmd_opts.h"
 #include "utils.h"
 #include "log_msg.h"
 
-#if FIREWALL_IPTABLES
+#if FIREWALL_FIREWALLD
+  #include "fw_util_firewalld.h"
+#elif FIREWALL_IPTABLES
   #include "fw_util_iptables.h"
 #endif
 
 /* Check to see if an integer variable has a value that is within a
  * specific range
 */
-static void
+static int
 range_check(fko_srv_options_t *opts, char *var, char *val, int low, int high)
 {
-    int     is_err;
+    int     is_err, rv;
 
-    strtol_wrapper(val, low, high, NO_EXIT_UPON_ERR, &is_err);
+    rv = strtol_wrapper(val, low, high, NO_EXIT_UPON_ERR, &is_err);
     if(is_err != FKO_SUCCESS)
     {
         log_msg(LOG_ERR, "[*] var %s value '%s' not in the range %d-%d",
@@ -55,7 +57,7 @@ range_check(fko_srv_options_t *opts, char *var, char *val, int low, int high)
         clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
     }
 
-    return;
+    return rv;
 }
 
 /* Take an index and a string value. malloc the space for the value
@@ -92,7 +94,7 @@ set_config_entry(fko_srv_options_t *opts, const int var_ndx, const char *value)
     */
     space_needed = strlen(value) + 1;
 
-    opts->config[var_ndx] = malloc(space_needed);
+    opts->config[var_ndx] = calloc(1, space_needed);
 
     if(opts->config[var_ndx] == NULL)
     {
@@ -140,14 +142,30 @@ validate_int_var_ranges(fko_srv_options_t *opts)
     int     is_err = FKO_SUCCESS;
 #endif
 
-    range_check(opts, "PCAP_LOOP_SLEEP", opts->config[CONF_PCAP_LOOP_SLEEP],
-        1, RCHK_MAX_PCAP_LOOP_SLEEP);
-    range_check(opts, "MAX_SPA_PACKET_AGE", opts->config[CONF_MAX_SPA_PACKET_AGE],
-        1, RCHK_MAX_SPA_PACKET_AGE);
-    range_check(opts, "MAX_SNIFF_BYTES", opts->config[CONF_MAX_SNIFF_BYTES],
-        1, RCHK_MAX_SNIFF_BYTES);
-    range_check(opts, "TCPSERV_PORT", opts->config[CONF_TCPSERV_PORT],
-        1, RCHK_MAX_TCPSERV_PORT);
+    opts->pcap_loop_sleep = range_check(opts,
+            "PCAP_LOOP_SLEEP", opts->config[CONF_PCAP_LOOP_SLEEP],
+            1, RCHK_MAX_PCAP_LOOP_SLEEP);
+    opts->pcap_dispatch_count = range_check(opts,
+            "PCAP_DISPATCH_COUNT", opts->config[CONF_PCAP_DISPATCH_COUNT],
+            1, RCHK_MAX_PCAP_DISPATCH_COUNT);
+    opts->max_spa_packet_age = range_check(opts,
+            "MAX_SPA_PACKET_AGE", opts->config[CONF_MAX_SPA_PACKET_AGE],
+            1, RCHK_MAX_SPA_PACKET_AGE);
+    opts->max_sniff_bytes = range_check(opts,
+            "MAX_SNIFF_BYTES", opts->config[CONF_MAX_SNIFF_BYTES],
+            1, RCHK_MAX_SNIFF_BYTES);
+    opts->rules_chk_threshold = range_check(opts,
+            "RULES_CHECK_THRESHOLD", opts->config[CONF_RULES_CHECK_THRESHOLD],
+            0, RCHK_MAX_RULES_CHECK_THRESHOLD);
+    opts->tcpserv_port = range_check(opts,
+            "TCPSERV_PORT", opts->config[CONF_TCPSERV_PORT],
+            1, RCHK_MAX_TCPSERV_PORT);
+    opts->udpserv_port = range_check(opts,
+            "UDPSERV_PORT", opts->config[CONF_UDPSERV_PORT],
+            1, RCHK_MAX_UDPSERV_PORT);
+    opts->udpserv_select_timeout = range_check(opts,
+            "UDPSERV_SELECT_TIMEOUT", opts->config[CONF_UDPSERV_SELECT_TIMEOUT],
+            1, RCHK_MAX_UDPSERV_SELECT_TIMEOUT);
 
 #if FIREWALL_IPFW
     range_check(opts, "IPFW_START_RULE_NUM", opts->config[CONF_IPFW_START_RULE_NUM],
@@ -194,6 +212,69 @@ validate_int_var_ranges(fko_srv_options_t *opts)
     return;
 }
 
+/**
+ * @brief Generate Rijndael + HMAC keys from /dev/urandom (base64 encoded).
+ *
+ * @param options FKO command line option structure
+ */
+static void
+generate_keys(fko_srv_options_t *options)
+{
+    char key_base64[MAX_B64_KEY_LEN+1];
+    char hmac_key_base64[MAX_B64_KEY_LEN+1];
+
+    FILE  *key_gen_file_ptr = NULL;
+    int res;
+
+    /* Set defaults and validate for --key-gen mode
+    */
+    if(options->key_len == 0)
+        options->key_len = FKO_DEFAULT_KEY_LEN;
+
+    if(options->hmac_key_len == 0)
+        options->hmac_key_len = FKO_DEFAULT_HMAC_KEY_LEN;
+
+    if(options->hmac_type == 0)
+        options->hmac_type = FKO_DEFAULT_HMAC_MODE;
+
+    /* Zero out the key buffers */
+    memset(key_base64, 0x00, sizeof(key_base64));
+    memset(hmac_key_base64, 0x00, sizeof(hmac_key_base64));
+
+    /* Generate the key through libfko */
+    res = fko_key_gen(key_base64, options->key_len,
+            hmac_key_base64, options->hmac_key_len,
+            options->hmac_type);
+
+    if(res != FKO_SUCCESS)
+    {
+        log_msg(LOG_ERR, "%s: fko_key_gen: Error %i - %s",
+            MY_NAME, res, fko_errstr(res));
+        clean_exit(options, NO_FW_CLEANUP, EXIT_FAILURE);
+    }
+
+    if(options->key_gen_file[0] != '\0')
+    {
+        if ((key_gen_file_ptr = fopen(options->key_gen_file, "w")) == NULL)
+        {
+            log_msg(LOG_ERR, "Unable to create key gen file: %s: %s",
+                options->key_gen_file, strerror(errno));
+            clean_exit(options, NO_FW_CLEANUP, EXIT_FAILURE);
+        }
+        fprintf(key_gen_file_ptr, "KEY_BASE64: %s\nHMAC_KEY_BASE64: %s\n",
+            key_base64, hmac_key_base64);
+        fclose(key_gen_file_ptr);
+        fprintf(stdout, "[+] Wrote Rijndael and HMAC keys to: %s",
+            options->key_gen_file);
+    }
+    else
+    {
+        fprintf(stdout, "KEY_BASE64: %s\nHMAC_KEY_BASE64: %s\n",
+                key_base64, hmac_key_base64);
+    }
+    clean_exit(options, NO_FW_CLEANUP, EXIT_SUCCESS);
+}
+
 /* Parse the config file...
 */
 static void
@@ -256,13 +337,6 @@ parse_config_file(fko_srv_options_t *opts, const char *config_file)
             continue;
         }
 
-        /*
-        fprintf(stderr,
-            "CONF FILE: %s, LINE: %s\tVar: %s, Val: '%s'\n",
-            config_file, conf_line_buf, var, val
-        );
-        */
-
         good_ent = 0;
         for(i=0; i<NUMBER_OF_CONFIG_ENTRIES; i++)
         {
@@ -281,6 +355,15 @@ parse_config_file(fko_srv_options_t *opts, const char *config_file)
                             strlcpy(val, opts->config[cndx], sizeof(val));
                             strlcat(val, tmp2, sizeof(val));
                         }
+                        else
+                        {
+                            /* We didn't map the embedded variable to a valid
+                             * config parameter
+                            */
+                            log_msg(LOG_ERR,
+                                "[*] Invalid embedded variable in: '%s'", val);
+                            break;
+                        }
                     }
                 }
 
@@ -350,7 +433,6 @@ validate_options(fko_srv_options_t *opts)
         if(tmp_path[strlen(tmp_path)-1] != '/')
             strlcat(tmp_path, "/", sizeof(tmp_path));
 
-
 #if USE_FILE_CACHE
         strlcat(tmp_path, DEF_DIGEST_CACHE_FILENAME, sizeof(tmp_path));
         set_config_entry(opts, CONF_DIGEST_FILE, tmp_path);
@@ -386,6 +468,13 @@ validate_options(fko_srv_options_t *opts)
         set_config_entry(opts, CONF_PCAP_LOOP_SLEEP,
             DEF_PCAP_LOOP_SLEEP);
 
+    /* Control whether to exit if the interface where we're sniffing
+     * goes down.
+    */
+    if(opts->config[CONF_EXIT_AT_INTF_DOWN] == NULL)
+        set_config_entry(opts, CONF_EXIT_AT_INTF_DOWN,
+            DEF_EXIT_AT_INTF_DOWN);
+
     /* PCAP Filter.
     */
     if(opts->config[CONF_PCAP_FILTER] == NULL)
@@ -413,19 +502,175 @@ validate_options(fko_srv_options_t *opts)
         set_config_entry(opts, CONF_MAX_SPA_PACKET_AGE,
             DEF_MAX_SPA_PACKET_AGE);
 
-
     /* Enable digest persistence.
     */
     if(opts->config[CONF_ENABLE_DIGEST_PERSISTENCE] == NULL)
         set_config_entry(opts, CONF_ENABLE_DIGEST_PERSISTENCE,
             DEF_ENABLE_DIGEST_PERSISTENCE);
 
+    /* Set firewall rule "deep" collection interval - this allows
+     * fwknopd to remove rules with proper _exp_<time> expiration
+     * times even when added by a different program.
+    */
+    if(opts->config[CONF_RULES_CHECK_THRESHOLD] == NULL)
+        set_config_entry(opts, CONF_RULES_CHECK_THRESHOLD,
+            DEF_RULES_CHECK_THRESHOLD);
+
+    /* Enable destination rule.
+    */
+    if(opts->config[CONF_ENABLE_DESTINATION_RULE] == NULL)
+        set_config_entry(opts, CONF_ENABLE_DESTINATION_RULE,
+            DEF_ENABLE_DESTINATION_RULE);
+
     /* Max sniff bytes.
     */
     if(opts->config[CONF_MAX_SNIFF_BYTES] == NULL)
         set_config_entry(opts, CONF_MAX_SNIFF_BYTES, DEF_MAX_SNIFF_BYTES);
 
-#if FIREWALL_IPTABLES
+#if FIREWALL_FIREWALLD
+    /* Enable FIREWD forwarding.
+    */
+    if(opts->config[CONF_ENABLE_FIREWD_FORWARDING] == NULL)
+        set_config_entry(opts, CONF_ENABLE_FIREWD_FORWARDING,
+            DEF_ENABLE_FIREWD_FORWARDING);
+
+    /* Enable FIREWD local NAT.
+    */
+    if(opts->config[CONF_ENABLE_FIREWD_LOCAL_NAT] == NULL)
+        set_config_entry(opts, CONF_ENABLE_FIREWD_LOCAL_NAT,
+            DEF_ENABLE_FIREWD_LOCAL_NAT);
+
+    /* Enable FIREWD SNAT.
+    */
+    if(opts->config[CONF_ENABLE_FIREWD_SNAT] == NULL)
+        set_config_entry(opts, CONF_ENABLE_FIREWD_SNAT,
+            DEF_ENABLE_FIREWD_SNAT);
+
+    /* Make sure we have a valid IP if SNAT is enabled
+    */
+    if(strncasecmp(opts->config[CONF_ENABLE_FIREWD_SNAT], "Y", 1) == 0)
+    {
+        /* Note that fw_config_init() will set use_masquerade if necessary
+        */
+        if(opts->config[CONF_SNAT_TRANSLATE_IP] != NULL)
+        {
+            if(! is_valid_ipv4_addr(opts->config[CONF_SNAT_TRANSLATE_IP], strlen(opts->config[CONF_SNAT_TRANSLATE_IP])))
+            {
+                log_msg(LOG_ERR,
+                    "Invalid IPv4 addr for SNAT_TRANSLATE_IP"
+                );
+                clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+            }
+        }
+    }
+
+    /* Enable FIREWD OUTPUT.
+    */
+    if(opts->config[CONF_ENABLE_FIREWD_OUTPUT] == NULL)
+        set_config_entry(opts, CONF_ENABLE_FIREWD_OUTPUT,
+            DEF_ENABLE_FIREWD_OUTPUT);
+
+    /* Flush FIREWD at init.
+    */
+    if(opts->config[CONF_FLUSH_FIREWD_AT_INIT] == NULL)
+        set_config_entry(opts, CONF_FLUSH_FIREWD_AT_INIT, DEF_FLUSH_FIREWD_AT_INIT);
+
+    /* Flush FIREWD at exit.
+    */
+    if(opts->config[CONF_FLUSH_FIREWD_AT_EXIT] == NULL)
+        set_config_entry(opts, CONF_FLUSH_FIREWD_AT_EXIT, DEF_FLUSH_FIREWD_AT_EXIT);
+
+    /* FIREWD input access.
+    */
+    if(opts->config[CONF_FIREWD_INPUT_ACCESS] == NULL)
+        set_config_entry(opts, CONF_FIREWD_INPUT_ACCESS,
+            DEF_FIREWD_INPUT_ACCESS);
+
+    if(validate_firewd_chain_conf(opts->config[CONF_FIREWD_INPUT_ACCESS]) != 1)
+    {
+        log_msg(LOG_ERR,
+            "Invalid FIREWD_INPUT_ACCESS specification, see fwknopd.conf comments"
+        );
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+    }
+
+    /* FIREWD output access.
+    */
+    if(opts->config[CONF_FIREWD_OUTPUT_ACCESS] == NULL)
+        set_config_entry(opts, CONF_FIREWD_OUTPUT_ACCESS,
+            DEF_FIREWD_OUTPUT_ACCESS);
+
+    if(validate_firewd_chain_conf(opts->config[CONF_FIREWD_OUTPUT_ACCESS]) != 1)
+    {
+        log_msg(LOG_ERR,
+            "Invalid FIREWD_OUTPUT_ACCESS specification, see fwknopd.conf comments"
+        );
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+    }
+
+    /* FIREWD forward access.
+    */
+    if(opts->config[CONF_FIREWD_FORWARD_ACCESS] == NULL)
+        set_config_entry(opts, CONF_FIREWD_FORWARD_ACCESS,
+            DEF_FIREWD_FORWARD_ACCESS);
+
+    if(validate_firewd_chain_conf(opts->config[CONF_FIREWD_FORWARD_ACCESS]) != 1)
+    {
+        log_msg(LOG_ERR,
+            "Invalid FIREWD_FORWARD_ACCESS specification, see fwknopd.conf comments"
+        );
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+    }
+
+    /* FIREWD dnat access.
+    */
+    if(opts->config[CONF_FIREWD_DNAT_ACCESS] == NULL)
+        set_config_entry(opts, CONF_FIREWD_DNAT_ACCESS,
+            DEF_FIREWD_DNAT_ACCESS);
+
+    if(validate_firewd_chain_conf(opts->config[CONF_FIREWD_DNAT_ACCESS]) != 1)
+    {
+        log_msg(LOG_ERR,
+            "Invalid FIREWD_DNAT_ACCESS specification, see fwknopd.conf comments"
+        );
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+    }
+
+    /* FIREWD snat access.
+    */
+    if(opts->config[CONF_FIREWD_SNAT_ACCESS] == NULL)
+        set_config_entry(opts, CONF_FIREWD_SNAT_ACCESS,
+            DEF_FIREWD_SNAT_ACCESS);
+
+    if(validate_firewd_chain_conf(opts->config[CONF_FIREWD_SNAT_ACCESS]) != 1)
+    {
+        log_msg(LOG_ERR,
+            "Invalid FIREWD_SNAT_ACCESS specification, see fwknopd.conf comments"
+        );
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+    }
+
+    /* FIREWD masquerade access.
+    */
+    if(opts->config[CONF_FIREWD_MASQUERADE_ACCESS] == NULL)
+        set_config_entry(opts, CONF_FIREWD_MASQUERADE_ACCESS,
+            DEF_FIREWD_MASQUERADE_ACCESS);
+
+    if(validate_firewd_chain_conf(opts->config[CONF_FIREWD_MASQUERADE_ACCESS]) != 1)
+    {
+        log_msg(LOG_ERR,
+            "Invalid FIREWD_MASQUERADE_ACCESS specification, see fwknopd.conf comments"
+        );
+        clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+    }
+
+    /* Check for the firewalld 'comment' match at init time
+    */
+    if(opts->config[CONF_ENABLE_FIREWD_COMMENT_CHECK] == NULL)
+        set_config_entry(opts, CONF_ENABLE_FIREWD_COMMENT_CHECK,
+            DEF_ENABLE_FIREWD_COMMENT_CHECK);
+
+#elif FIREWALL_IPTABLES
     /* Enable IPT forwarding.
     */
     if(opts->config[CONF_ENABLE_IPT_FORWARDING] == NULL)
@@ -452,7 +697,7 @@ validate_options(fko_srv_options_t *opts)
         */
         if(opts->config[CONF_SNAT_TRANSLATE_IP] != NULL)
         {
-            if(! is_valid_ipv4_addr(opts->config[CONF_SNAT_TRANSLATE_IP]))
+            if(! is_valid_ipv4_addr(opts->config[CONF_SNAT_TRANSLATE_IP], strlen(opts->config[CONF_SNAT_TRANSLATE_IP])))
             {
                 log_msg(LOG_ERR,
                     "Invalid IPv4 addr for SNAT_TRANSLATE_IP"
@@ -634,11 +879,33 @@ validate_options(fko_srv_options_t *opts)
 
 #endif /* FIREWALL type */
 
+    /* Disallow ENABLE_X_FORWARDED_FOR by default*/
+    if(opts->config[CONF_ENABLE_X_FORWARDED_FOR] == NULL)
+        set_config_entry(opts, CONF_ENABLE_X_FORWARDED_FOR, DEF_ENABLE_X_FORWARDED_FOR);
+
+    /* Prepend firewall rules*/
+    if(opts->config[CONF_ENABLE_RULE_PREPEND] == NULL)
+        set_config_entry(opts, CONF_ENABLE_RULE_PREPEND, DEF_ENABLE_RULE_PREPEND);
+
+    /* NAT DNS enabled*/
+    if(opts->config[CONF_ENABLE_NAT_DNS] == NULL)
+        set_config_entry(opts, CONF_ENABLE_NAT_DNS, DEF_ENABLE_NAT_DNS);
+
     /* GPG Home dir.
     */
     if(opts->config[CONF_GPG_HOME_DIR] == NULL)
         set_config_entry(opts, CONF_GPG_HOME_DIR, DEF_GPG_HOME_DIR);
 
+    /* GPG executable
+    */
+    if(opts->config[CONF_GPG_EXE] == NULL)
+        set_config_entry(opts, CONF_GPG_EXE, DEF_GPG_EXE);
+
+    /* sudo executable
+    */
+    if(opts->config[CONF_SUDO_EXE] == NULL)
+        set_config_entry(opts, CONF_SUDO_EXE, DEF_SUDO_EXE);
+
     /* Enable SPA over HTTP.
     */
     if(opts->config[CONF_ENABLE_SPA_OVER_HTTP] == NULL)
@@ -655,6 +922,74 @@ validate_options(fko_srv_options_t *opts)
     if(opts->config[CONF_TCPSERV_PORT] == NULL)
         set_config_entry(opts, CONF_TCPSERV_PORT, DEF_TCPSERV_PORT);
 
+#if USE_LIBNETFILTER_QUEUE
+    /* Enable NFQ Capture
+    */
+    if(opts->config[CONF_ENABLE_NFQ_CAPTURE] == NULL)
+        set_config_entry(opts, CONF_ENABLE_NFQ_CAPTURE, DEF_ENABLE_NFQ_CAPTURE);
+
+    if((strncasecmp(opts->config[CONF_ENABLE_NFQ_CAPTURE], "Y", 1) == 0) &&
+            !opts->enable_nfq_capture)
+    {
+        opts->enable_nfq_capture = 1;
+    }
+
+    /* NFQ Interface
+    */
+    if(opts->config[CONF_NFQ_INTERFACE] == NULL)
+        set_config_entry(opts, CONF_NFQ_INTERFACE, DEF_NFQ_INTERFACE);
+
+    /* NFQ port.
+    */
+    if(opts->config[CONF_NFQ_PORT] == NULL)
+        set_config_entry(opts, CONF_NFQ_PORT, DEF_NFQ_PORT);
+
+    /* NFQ Queue Number
+    */
+    if(opts->config[CONF_NFQ_QUEUE_NUMBER] == NULL)
+        set_config_entry(opts, CONF_NFQ_QUEUE_NUMBER,
+            DEF_NFQ_QUEUE_NUMBER);
+
+    /* NFQ Chain
+    */
+    if(opts->config[CONF_NFQ_CHAIN] == NULL)
+        set_config_entry(opts, CONF_NFQ_CHAIN, DEF_NFQ_CHAIN);
+
+    /* NFQ Table
+    */
+    if(opts->config[CONF_NFQ_TABLE] == NULL)
+        set_config_entry(opts, CONF_NFQ_TABLE, DEF_NFQ_TABLE);
+
+    /* NFQ loop delay
+    */
+    if(opts->config[CONF_NFQ_LOOP_SLEEP] == NULL)
+        set_config_entry(opts, CONF_NFQ_LOOP_SLEEP, DEF_CONF_NFQ_LOOP_SLEEP);
+#endif
+
+    /* Enable UDP server.
+    */
+    if(opts->config[CONF_ENABLE_UDP_SERVER] == NULL)
+    {
+        if((strncasecmp(DEF_ENABLE_UDP_SERVER, "Y", 1) == 0) &&
+                !opts->enable_udp_server)
+        {
+            log_msg(LOG_ERR, "pcap capture not compiled in, forcing UDP server mode");
+            opts->enable_udp_server = 1;
+        }
+        set_config_entry(opts, CONF_ENABLE_UDP_SERVER, DEF_ENABLE_UDP_SERVER);
+    }
+
+    /* UDP Server port.
+    */
+    if(opts->config[CONF_UDPSERV_PORT] == NULL)
+        set_config_entry(opts, CONF_UDPSERV_PORT, DEF_UDPSERV_PORT);
+
+    /* UDP server select() timeout in microseconds
+    */
+    if(opts->config[CONF_UDPSERV_SELECT_TIMEOUT] == NULL)
+        set_config_entry(opts, CONF_UDPSERV_SELECT_TIMEOUT,
+            DEF_UDPSERV_SELECT_TIMEOUT);
+
     /* Syslog identity.
     */
     if(opts->config[CONF_SYSLOG_IDENTITY] == NULL)
@@ -740,10 +1075,76 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
     */
     optind = 0;
 
-    /* First, scan the command-line args for -h/--help or an alternate
+    /* Enable access.conf parsing (in case this is a re-config)
+    */
+    enable_acc_stanzas_init();
+
+    /* First, scan the command-line args to see if we are in key-generation
+     * mode. This is independent of config parsing and other operations, so
+     * it is done as the very first thing. Also handle printing of the fwknop
+     * version string since we don't need to parse a config for this.
+    */
+    while ((cmd_arg = getopt_long(argc, argv,
+            GETOPTS_OPTION_STRING, cmd_opts, &index)) != -1) {
+
+        switch(cmd_arg) {
+            case 'V':
+                fprintf(stdout, "fwknopd server %s, compiled for firewall bin: %s\n",
+                        MY_VERSION, FIREWALL_EXE);
+                clean_exit(opts, NO_FW_CLEANUP, EXIT_SUCCESS);
+            case 'k':
+                opts->key_gen = 1;
+                break;
+            case KEY_GEN_FILE:
+                opts->key_gen = 1;
+                strlcpy(opts->key_gen_file, optarg, sizeof(opts->key_gen_file));
+                break;
+            case KEY_LEN:  /* used in --key-gen mode only */
+                opts->key_len = strtol_wrapper(optarg, 1,
+                        MAX_KEY_LEN, NO_EXIT_UPON_ERR, &is_err);
+                if(is_err != FKO_SUCCESS)
+                {
+                    log_msg(LOG_ERR,
+                            "Invalid key length '%s', must be in [%d-%d]",
+                            optarg, 1, MAX_KEY_LEN);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
+                break;
+            case HMAC_DIGEST_TYPE:  /* used in --key-gen mode only */
+                if((opts->hmac_type = hmac_digest_strtoint(optarg)) < 0)
+                {
+                    log_msg(LOG_ERR,
+                        "* Invalid hmac digest type: %s, use {md5,sha1,sha256,sha384,sha512}",
+                        optarg);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
+                break;
+            case HMAC_KEY_LEN:  /* used in --key-gen mode only */
+                opts->hmac_key_len = strtol_wrapper(optarg, 1,
+                        MAX_KEY_LEN, NO_EXIT_UPON_ERR, &is_err);
+                if(is_err != FKO_SUCCESS)
+                {
+                    log_msg(LOG_ERR,
+                            "Invalid hmac key length '%s', must be in [%d-%d]",
+                            optarg, 1, MAX_KEY_LEN);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
+                break;
+        }
+    }
+
+    if(opts->key_gen)
+        generate_keys(opts); /* this function exits */
+
+    /* Reset the options index so we can run through them again.
+    */
+    optind = 0;
+
+    /* Now, scan the command-line args for -h/--help or an alternate
      * configuration file. If we find an alternate config file, use it,
-     * otherwise use the default.  We also grab any override config files
-     * as well.
+     * otherwise use the default. We also grab any override config files
+     * as well. In addition, we handle key generation here since this is
+     * independent of configuration parsing.
     */
     while ((cmd_arg = getopt_long(argc, argv,
             GETOPTS_OPTION_STRING, cmd_opts, &index)) != -1) {
@@ -754,7 +1155,6 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
             case 'h':
                 usage();
                 clean_exit(opts, NO_FW_CLEANUP, EXIT_SUCCESS);
-                break;
 
             /* Look for configuration file arg.
             */
@@ -851,8 +1251,44 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
             GETOPTS_OPTION_STRING, cmd_opts, &index)) != -1) {
 
         switch(cmd_arg) {
+            case 'A':
+#if AFL_FUZZING
+                opts->afl_fuzzing = 1;
+#else
+                log_msg(LOG_ERR, "[*] fwknopd not compiled with AFL fuzzing support");
+                clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+#endif
+                break;
+            case AFL_PKT_FILE:
+#if AFL_FUZZING
+                opts->afl_fuzzing = 1;
+                set_config_entry(opts, CONF_AFL_PKT_FILE, optarg);
+#else
+                log_msg(LOG_ERR, "[*] fwknopd not compiled with AFL fuzzing support");
+                clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+#endif
+                break;
             case 'a':
-                set_config_entry(opts, CONF_ACCESS_FILE, optarg);
+                if (is_valid_file(optarg))
+                    set_config_entry(opts, CONF_ACCESS_FILE, optarg);
+                else
+                {
+                    log_msg(LOG_ERR,
+                        "[*] Invalid access.conf file path '%s'", optarg);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
+                break;
+            case ACCESS_FOLDER:
+                chop_char(optarg, PATH_SEP);
+                if (is_valid_dir(optarg))
+                    set_config_entry(opts, CONF_ACCESS_FOLDER, optarg);
+                else
+                {
+                    log_msg(LOG_ERR,
+                        "[*] Invalid access folder directory '%s' could not lstat(), does not exist, or too large?",
+                        optarg);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
                 break;
             case 'c':
                 /* This was handled earlier */
@@ -878,9 +1314,28 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
             case 'D':
                 opts->dump_config = 1;
                 break;
+            case DUMP_SERVER_ERR_CODES:
+                dump_server_errors();
+                clean_exit(opts, NO_FW_CLEANUP, EXIT_SUCCESS);
+            case EXIT_AFTER_PARSE_CONFIG:
+                opts->exit_after_parse_config = 1;
+                opts->foreground = 1;
+                break;
+            case EXIT_VALIDATE_DIGEST_CACHE:
+                opts->exit_parse_digest_cache = 1;
+                opts->foreground = 1;
+                break;
             case 'f':
                 opts->foreground = 1;
                 break;
+            case FAULT_INJECTION_TAG:
+#if HAVE_LIBFIU
+                set_config_entry(opts, CONF_FAULT_INJECTION_TAG, optarg);
+#else
+                log_msg(LOG_ERR, "[*] fwknopd not compiled with libfiu support");
+                clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+#endif
+                break;
             case FW_LIST:
                 opts->fw_list = 1;
                 break;
@@ -891,15 +1346,25 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
             case FW_FLUSH:
                 opts->fw_flush = 1;
                 break;
-            case GPG_HOME_DIR:
-                if (is_valid_dir(optarg))
-                {
-                    set_config_entry(opts, CONF_GPG_HOME_DIR, optarg);
-                }
+            case GPG_EXE_PATH:
+                if (is_valid_exe(optarg))
+                    set_config_entry(opts, CONF_GPG_EXE, optarg);
                 else
                 {
                     log_msg(LOG_ERR,
-                        "[*] Directory '%s' could not stat()/does not exist?",
+                        "[*] gpg path '%s' could not lstat()/not executable?",
+                        optarg);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
+                break;
+            case GPG_HOME_DIR:
+                chop_char(optarg, PATH_SEP);
+                if (is_valid_dir(optarg))
+                    set_config_entry(opts, CONF_GPG_HOME_DIR, optarg);
+                else
+                {
+                    log_msg(LOG_ERR,
+                        "[*] gpg home directory '%s' could not lstat(), does not exist, or too large?",
                         optarg);
                     clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
                 }
@@ -907,12 +1372,23 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
             case 'i':
                 set_config_entry(opts, CONF_PCAP_INTF, optarg);
                 break;
+            case FIREWD_DISABLE_CHECK_SUPPORT:
+                opts->firewd_disable_check_support = 1;
+                break;
+            case IPT_DISABLE_CHECK_SUPPORT:
+                opts->ipt_disable_check_support = 1;
+                break;
             case 'K':
                 opts->kill = 1;
                 break;
             case 'l':
                 set_config_entry(opts, CONF_LOCALE, optarg);
                 break;
+#if USE_LIBNETFILTER_QUEUE
+            case 'n':
+                opts->enable_nfq_capture = 1;
+                break;
+#endif
             case 'O':
                 /* This was handled earlier */
                 break;
@@ -923,7 +1399,14 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
                 set_config_entry(opts, CONF_PCAP_FILTER, optarg);
                 break;
             case PCAP_FILE:
-                set_config_entry(opts, CONF_PCAP_FILE, optarg);
+                if (is_valid_file(optarg))
+                    set_config_entry(opts, CONF_PCAP_FILE, optarg);
+                else
+                {
+                    log_msg(LOG_ERR,
+                        "[*] Invalid pcap file path '%s'", optarg);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
                 break;
             case ENABLE_PCAP_ANY_DIRECTION:
                 opts->pcap_any_direction = 1;
@@ -934,9 +1417,29 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
             case 'R':
                 opts->restart = 1;
                 break;
+            case 'r':
+                set_config_entry(opts, CONF_FWKNOP_RUN_DIR, optarg);
+                break;
             case 'S':
                 opts->status = 1;
                 break;
+            case SUDO_EXE_PATH:
+                if (is_valid_exe(optarg))
+                    set_config_entry(opts, CONF_SUDO_EXE, optarg);
+                else
+                {
+                    log_msg(LOG_ERR,
+                        "[*] sudo path '%s' could not stat()/not executable?",
+                        optarg);
+                    clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
+                }
+                break;
+            case 't':
+                opts->test = 1;
+                break;
+            case 'U':
+                opts->enable_udp_server = 1;
+                break;
             /* Verbosity level */
             case 'v':
                 opts->verbose++;
@@ -944,10 +1447,6 @@ config_init(fko_srv_options_t *opts, int argc, char **argv)
             case SYSLOG_ENABLE:
                 opts->syslog_enable = 1;
                 break;
-            case 'V':
-                fprintf(stdout, "fwknopd server %s\n", MY_VERSION);
-                clean_exit(opts, NO_FW_CLEANUP, EXIT_SUCCESS);
-                break;
             default:
                 usage();
                 clean_exit(opts, NO_FW_CLEANUP, EXIT_FAILURE);
@@ -991,21 +1490,25 @@ usage(void)
             MY_NAME, MY_VERSION, MY_DESC);
     fprintf(stdout,
       "Usage: fwknopd [options]\n\n"
-      " -h, --help              - Print this usage message and exit.\n"
       " -a, --access-file       - Specify an alternate access.conf file.\n"
+      "     --access-folder     - Specify an access.conf folder.  All .conf\n"
+      "                           files in this folder will be processed.\n"
       " -c, --config-file       - Specify an alternate configuration file.\n"
-      " -C, --packet-limit      - Limit the number of candidate SPA packets to\n"
-      "                           process and exit when this limit is reached.\n"
-      " -d, --digest-file       - Specify an alternate digest.cache file.\n"
-      " -D, --dump-config       - Dump the current fwknop configuration values.\n"
       " -f, --foreground        - Run fwknopd in the foreground (do not become\n"
       "                           a background daemon).\n"
       " -i, --interface         - Specify interface to listen for incoming SPA\n"
       "                           packets.\n"
+      " -C, --packet-limit      - Limit the number of candidate SPA packets to\n"
+      "                           process and exit when this limit is reached.\n"
+      " -d, --digest-file       - Specify an alternate digest.cache file.\n"
+      " -D, --dump-config       - Dump the current fwknop configuration values.\n"
       " -K, --kill              - Kill the currently running fwknopd.\n"
-      "     --gpg-home-dir      - Specify the GPG home directory.\n"
       " -l, --locale            - Provide a locale setting other than the system\n"
       "                           default.\n"
+#if USE_LIBNETFILTER_QUEUE
+      " -n, --nfq-capture       - Capture packets using libnetfilter_queue (falls\n"
+      "                           back to UDP server mode if not used).\n"
+#endif
       " -O, --override-config   - Specify a file with configuration entries that will\n"
       "                           overide those in fwknopd.conf\n"
       " -p, --pid-file          - Specify an alternate fwknopd.pid file.\n"
@@ -1013,19 +1516,49 @@ usage(void)
       "                           override the PCAP_FILTER variable in fwknopd.conf.\n"
       " -R, --restart           - Force the currently running fwknopd to restart.\n"
       "     --rotate-digest-cache\n"
+      "                         - Rotate the digest cache file by renaming the file\n"
+      "                           to the same path with the -old suffix.\n"
+      " -r, --run-dir           - Set path to local state run directory.\n"
       "                         - Rotate the digest cache file by renaming it to\n"
       "                           '<name>-old', and starting a new one.\n"
       " -S, --status            - Display the status of any running fwknopd process.\n"
+      " -t, --test              - Test mode, process SPA packets but do not make any\n"
+      "                           firewall modifications.\n"
+      " -U, --udp-server        - Set UDP server mode.\n"
       " -v, --verbose           - Set verbose mode.\n"
       "     --syslog-enable     - Allow messages to be sent to syslog even if the\n"
       "                           foreground mode is set.\n"
       " -V, --version           - Print version number.\n"
+      " -A, --afl-fuzzing       - Run in American Fuzzy Lop (AFL) fuzzing mode so\n"
+      "                           that plaintext SPA packets are accepted via stdin.\n"
+      " -h, --help              - Print this usage message and exit.\n"
+      " --dump-serv-err-codes   - List all server error codes (only needed by the\n"
+      "                           test suite).\n"
+      " --exit-parse-config     - Parse config files and exit.\n"
+      " --exit-parse-digest-cache - Parse and validate digest cache  and exit.\n"
+      " --fault-injection-tag   - Enable a fault injection tag (only needed by the\n"
+      "                           test suite).\n"
+      " --pcap-file             - Read potential SPA packets from an existing pcap\n"
+      "                           file.\n"
+      " --pcap-any-direction    - By default fwknopd processes packets that are\n"
+      "                           sent to the sniffing interface, but this option\n"
+      "                           enables processing of packets that originate from\n"
+      "                           an interface (such as in a forwarding situation).\n"
       "     --fw-list           - List all firewall rules that fwknop has created\n"
       "                           and then exit.\n"
       "     --fw-list-all       - List all firewall rules in the complete policy,\n"
       "                           including those that have nothing to do with\n"
       "                           fwknop.\n"
       "     --fw-flush          - Flush all firewall rules created by fwknop.\n"
+      "     --gpg-home-dir      - Specify the GPG home directory (this is normally\n"
+      "                           done in the access.conf file).\n"
+      "     --gpg-exe           - Specify the path to GPG (this is normally done in\n"
+      "                           the access.conf file).\n"
+      "     --sudo-exe          - Specify the path to sudo (the default path is\n"
+      "                           /usr/bin/sudo).\n"
+      " --no-firewd-check-support\n"
+      "                         - Disable test for 'firewall-cmd ... -C' support.\n"
+      " --no-ipt-check-support  - Disable test for 'iptables -C' support.\n"
       "\n"
     );
 
