@@ -1,13 +1,12 @@
-/*
- ******************************************************************************
+/**
+ * \file server/fwknopd_common.h
  *
- * File:    fwknopd_common.h
- *
- * Author:  Damien Stuart
- *
- * Purpose: Header file for fwknopd source files.
- *
- * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Header file for fwknopd source files.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -41,7 +40,7 @@
   #include <sys/stat.h>
 #endif
 
-#if HAVE_LIBPCAP
+#if USE_LIBPCAP
   #include <pcap.h>
 #endif
 
@@ -70,7 +69,7 @@
   /* Our default run directory is based on LOCALSTATEDIR as set by the
    * configure script. This is where we put the PID and digest cache files.
   */
-  #define DEF_RUN_DIR       SYSRUNDIR"/run/"PACKAGE_NAME
+  #define DEF_RUN_DIR       SYSRUNDIR"/"PACKAGE_NAME
 #endif
 
 /* More Conf defaults
@@ -88,16 +87,49 @@
 #define DEF_PCAP_DISPATCH_COUNT         "100"
 #define DEF_PCAP_LOOP_SLEEP             "100000" /* a tenth of a second (in microseconds) */
 #define DEF_ENABLE_PCAP_ANY_DIRECTION   "N"
+#define DEF_EXIT_AT_INTF_DOWN           "Y"
 #define DEF_ENABLE_SPA_PACKET_AGING     "Y"
 #define DEF_MAX_SPA_PACKET_AGE          "120"
 #define DEF_ENABLE_DIGEST_PERSISTENCE   "Y"
+#define DEF_RULES_CHECK_THRESHOLD       "20"
 #define DEF_MAX_SNIFF_BYTES             "1500"
 #define DEF_GPG_HOME_DIR                "/root/.gnupg"
+#define DEF_ENABLE_X_FORWARDED_FOR      "N"
+#define DEF_ENABLE_RULE_PREPEND         "N"
+#define DEF_ENABLE_NAT_DNS              "Y"
+#ifdef  GPG_EXE
+  #define DEF_GPG_EXE                   GPG_EXE
+#else
+  #define DEF_GPG_EXE                   "/usr/bin/gpg"
+#endif
+#ifdef  SUDO_EXE
+  #define DEF_SUDO_EXE                   SUDO_EXE
+#else
+  #define DEF_SUDO_EXE                   "/usr/bin/sudo"
+#endif
 #define DEF_ENABLE_SPA_OVER_HTTP        "N"
 #define DEF_ENABLE_TCP_SERVER           "N"
 #define DEF_TCPSERV_PORT                "62201"
+#if USE_LIBPCAP
+  #define DEF_ENABLE_UDP_SERVER           "N"
+#else
+  #define DEF_ENABLE_UDP_SERVER           "Y"
+#endif
+#if USE_LIBNETFILTER_QUEUE
+  #define DEF_ENABLE_NFQ_CAPTURE          "N"
+  #define DEF_NFQ_INTERFACE               ""
+  #define DEF_NFQ_PORT                    "62201"
+  #define DEF_NFQ_TABLE                   "mangle"
+  #define DEF_NFQ_CHAIN                   "FWKNOP_NFQ"
+  #define DEF_NFQ_QUEUE_NUMBER            "1"
+  #define DEF_CONF_NFQ_LOOP_SLEEP         "500000" /* half a second (in microseconds) */
+
+#endif
+#define DEF_UDPSERV_PORT                "62201"
+#define DEF_UDPSERV_SELECT_TIMEOUT      "500000" /* half a second (in microseconds) */
 #define DEF_SYSLOG_IDENTITY             MY_NAME
 #define DEF_SYSLOG_FACILITY             "LOG_DAEMON"
+#define DEF_ENABLE_DESTINATION_RULE     "N"
 
 #define DEF_FW_ACCESS_TIMEOUT           30
 
@@ -107,17 +139,42 @@
 #define RCHK_MAX_SPA_PACKET_AGE         100000  /* seconds, can disable */
 #define RCHK_MAX_SNIFF_BYTES            (2 << 14)
 #define RCHK_MAX_TCPSERV_PORT           ((2 << 16) - 1)
+#define RCHK_MAX_UDPSERV_PORT           ((2 << 16) - 1)
+#define RCHK_MAX_UDPSERV_SELECT_TIMEOUT (2 << 22)
 #define RCHK_MAX_PCAP_DISPATCH_COUNT    (2 << 22)
-#define RCHK_MAX_FW_TIMEOUT             (2 << 22)
+#define RCHK_MAX_FW_TIMEOUT             (2 << 22) /* seconds */
+#define RCHK_MAX_CMD_CYCLE_TIMER        (2 << 22) /* seconds */
+#define RCHK_MIN_CMD_CYCLE_TIMER        1
+#define RCHK_MAX_RULES_CHECK_THRESHOLD  ((2 << 16) - 1)
+
+/* FirewallD-specific defines
+*/
+#if FIREWALL_FIREWALLD
+
+  #define DEF_FLUSH_FIREWD_AT_INIT         "Y"
+  #define DEF_FLUSH_FIREWD_AT_EXIT         "Y"
+  #define DEF_ENABLE_FIREWD_FORWARDING     "N"
+  #define DEF_ENABLE_FIREWD_LOCAL_NAT      "N"
+  #define DEF_ENABLE_FIREWD_SNAT           "N"
+  #define DEF_ENABLE_FIREWD_OUTPUT         "N"
+  #define DEF_ENABLE_FIREWD_COMMENT_CHECK  "Y"
+  #define DEF_FIREWD_INPUT_ACCESS          "ACCEPT, filter, INPUT, 1, FWKNOP_INPUT, 1"
+  #define DEF_FIREWD_OUTPUT_ACCESS         "ACCEPT, filter, OUTPUT, 1, FWKNOP_OUTPUT, 1"
+  #define DEF_FIREWD_FORWARD_ACCESS        "ACCEPT, filter, FORWARD, 1, FWKNOP_FORWARD, 1"
+  #define DEF_FIREWD_DNAT_ACCESS           "DNAT, nat, PREROUTING, 1, FWKNOP_PREROUTING, 1"
+  #define DEF_FIREWD_SNAT_ACCESS           "SNAT, nat, POSTROUTING, 1, FWKNOP_POSTROUTING, 1"
+  #define DEF_FIREWD_MASQUERADE_ACCESS     "MASQUERADE, nat, POSTROUTING, 1, FWKNOP_MASQUERADE, 1"
+
+  #define RCHK_MAX_FIREWD_RULE_NUM         (2 << 15)
 
 /* Iptables-specific defines
 */
-#if FIREWALL_IPTABLES
+#elif FIREWALL_IPTABLES
 
   #define DEF_FLUSH_IPT_AT_INIT         "Y"
   #define DEF_FLUSH_IPT_AT_EXIT         "Y"
   #define DEF_ENABLE_IPT_FORWARDING     "N"
-  #define DEF_ENABLE_IPT_LOCAL_NAT      "Y"
+  #define DEF_ENABLE_IPT_LOCAL_NAT      "N"
   #define DEF_ENABLE_IPT_SNAT           "N"
   #define DEF_ENABLE_IPT_OUTPUT         "N"
   #define DEF_ENABLE_IPT_COMMENT_CHECK  "Y"
@@ -126,7 +183,7 @@
   #define DEF_IPT_FORWARD_ACCESS        "ACCEPT, filter, FORWARD, 1, FWKNOP_FORWARD, 1"
   #define DEF_IPT_DNAT_ACCESS           "DNAT, nat, PREROUTING, 1, FWKNOP_PREROUTING, 1"
   #define DEF_IPT_SNAT_ACCESS           "SNAT, nat, POSTROUTING, 1, FWKNOP_POSTROUTING, 1"
-  #define DEF_IPT_MASQUERADE_ACCESS     "MASQUERADE, nat, POSTROUTING, 1, FWKNOP_POSTROUTING, 1"
+  #define DEF_IPT_MASQUERADE_ACCESS     "MASQUERADE, nat, POSTROUTING, 1, FWKNOP_MASQUERADE, 1"
 
   #define RCHK_MAX_IPT_RULE_NUM         (2 << 15)
 
@@ -166,7 +223,6 @@
 #define MAX_PCAP_FILTER_LEN     1024
 #define MAX_IFNAME_LEN          128
 #define MAX_SPA_PACKET_LEN      1500 /* --DSS check this? */
-#define MAX_HOSTNAME_LEN        64
 #define MAX_DECRYPTED_SPA_LEN   1024
 
 /* The minimum possible valid SPA data size.
@@ -191,15 +247,29 @@ enum {
     CONF_PCAP_DISPATCH_COUNT,
     CONF_PCAP_LOOP_SLEEP,
     CONF_ENABLE_PCAP_ANY_DIRECTION,
+    CONF_EXIT_AT_INTF_DOWN,
     CONF_MAX_SNIFF_BYTES,
     CONF_ENABLE_SPA_PACKET_AGING,
     CONF_MAX_SPA_PACKET_AGE,
     CONF_ENABLE_DIGEST_PERSISTENCE,
+    CONF_RULES_CHECK_THRESHOLD,
     CONF_CMD_EXEC_TIMEOUT,
     //CONF_BLACKLIST,
     CONF_ENABLE_SPA_OVER_HTTP,
     CONF_ENABLE_TCP_SERVER,
     CONF_TCPSERV_PORT,
+    CONF_ENABLE_UDP_SERVER,
+    CONF_UDPSERV_PORT,
+    CONF_UDPSERV_SELECT_TIMEOUT,
+#if USE_LIBNETFILTER_QUEUE
+    CONF_ENABLE_NFQ_CAPTURE,
+    CONF_NFQ_INTERFACE,
+    CONF_NFQ_PORT,
+    CONF_NFQ_TABLE,
+    CONF_NFQ_CHAIN,
+    CONF_NFQ_QUEUE_NUMBER,
+    CONF_NFQ_LOOP_SLEEP,
+#endif
     CONF_LOCALE,
     CONF_SYSLOG_IDENTITY,
     CONF_SYSLOG_FACILITY,
@@ -210,7 +280,26 @@ enum {
     //CONF_EXTERNAL_CMD_ALARM,
     //CONF_ENABLE_EXT_CMD_PREFIX,
     //CONF_EXT_CMD_PREFIX,
-#if FIREWALL_IPTABLES
+    CONF_ENABLE_X_FORWARDED_FOR,
+    CONF_ENABLE_DESTINATION_RULE,
+    CONF_ENABLE_RULE_PREPEND,
+    CONF_ENABLE_NAT_DNS,
+#if FIREWALL_FIREWALLD
+    CONF_ENABLE_FIREWD_FORWARDING,
+    CONF_ENABLE_FIREWD_LOCAL_NAT,
+    CONF_ENABLE_FIREWD_SNAT,
+    CONF_SNAT_TRANSLATE_IP,
+    CONF_ENABLE_FIREWD_OUTPUT,
+    CONF_FLUSH_FIREWD_AT_INIT,
+    CONF_FLUSH_FIREWD_AT_EXIT,
+    CONF_FIREWD_INPUT_ACCESS,
+    CONF_FIREWD_OUTPUT_ACCESS,
+    CONF_FIREWD_FORWARD_ACCESS,
+    CONF_FIREWD_DNAT_ACCESS,
+    CONF_FIREWD_SNAT_ACCESS,
+    CONF_FIREWD_MASQUERADE_ACCESS,
+    CONF_ENABLE_FIREWD_COMMENT_CHECK,
+#elif FIREWALL_IPTABLES
     CONF_ENABLE_IPT_FORWARDING,
     CONF_ENABLE_IPT_LOCAL_NAT,
     CONF_ENABLE_IPT_SNAT,
@@ -243,6 +332,7 @@ enum {
     CONF_FWKNOP_RUN_DIR,
     CONF_FWKNOP_CONF_DIR,
     CONF_ACCESS_FILE,
+    CONF_ACCESS_FOLDER,
     CONF_FWKNOP_PID_FILE,
 #if USE_FILE_CACHE
     CONF_DIGEST_FILE,
@@ -250,8 +340,14 @@ enum {
     CONF_DIGEST_DB_FILE,
 #endif
     CONF_GPG_HOME_DIR,
+    CONF_GPG_EXE,
+    CONF_SUDO_EXE,
     CONF_FIREWALL_EXE,
     CONF_VERBOSE,
+#if AFL_FUZZING
+    CONF_AFL_PKT_FILE,
+#endif
+    CONF_FAULT_INJECTION_TAG,
 
     NUMBER_OF_CONFIG_ENTRIES  /* Marks the end and number of entries */
 };
@@ -291,6 +387,8 @@ typedef struct acc_stanza
 {
     char                *source;
     acc_int_list_t      *source_list;
+    char                *destination;
+    acc_int_list_t      *destination_list;
     char                *open_ports;
     acc_port_list_t     *oport_list;
     char                *restrict_ports;
@@ -305,32 +403,46 @@ typedef struct acc_stanza
     unsigned char        use_rijndael;
     int                  fw_access_timeout;
     unsigned char        enable_cmd_exec;
+    unsigned char        enable_cmd_sudo_exec;
+    char                *cmd_sudo_exec_user;
+    char                *cmd_sudo_exec_group;
+    uid_t                cmd_sudo_exec_uid;
+    gid_t                cmd_sudo_exec_gid;
     char                *cmd_exec_user;
+    char                *cmd_exec_group;
+    char                *cmd_cycle_open;
+    char                *cmd_cycle_close;
+    unsigned char        cmd_cycle_do_close;
+    int                  cmd_cycle_timer;
     uid_t                cmd_exec_uid;
+    gid_t                cmd_exec_gid;
     char                *require_username;
     unsigned char        require_source_address;
     char                *gpg_home_dir;
+    char                *gpg_exe;
     char                *gpg_decrypt_id;
     char                *gpg_decrypt_pw;
     unsigned char        gpg_require_sig;
+    unsigned char        gpg_disable_sig;
     unsigned char        gpg_ignore_sig_error;
     unsigned char        use_gpg;
     unsigned char        gpg_allow_no_pw;
     char                *gpg_remote_id;
     acc_string_list_t   *gpg_remote_id_list;
+    char                *gpg_remote_fpr;
+    acc_string_list_t   *gpg_remote_fpr_list;
     time_t               access_expire_time;
     int                  expired;
     int                  encryption_mode;
 
-    /* DNAT parameters
+    /* NAT parameters
     */
     unsigned char        force_nat;
     char                *force_nat_ip;
     char                *force_nat_proto;
     unsigned int         force_nat_port;
-
-    /* SNAT parameters
-    */
+    unsigned char        forward_all;
+    unsigned char        disable_dnat;
     unsigned char        force_snat;
     char                *force_snat_ip;
     unsigned char        force_masquerade;
@@ -338,10 +450,67 @@ typedef struct acc_stanza
     struct acc_stanza   *next;
 } acc_stanza_t;
 
+/* A simple linked list of strings for command open/close cycles
+*/
+typedef struct cmd_cycle_list
+{
+    char                    src_ip[MAX_IPV4_STR_LEN];
+    char                   *close_cmd;
+    time_t                  expire;
+    int                     stanza_num;
+    struct cmd_cycle_list  *next;
+} cmd_cycle_list_t;
 
 /* Firewall-related data and types. */
 
-#if FIREWALL_IPTABLES
+#if FIREWALL_FIREWALLD
+  /* --DSS XXX: These are arbitrary. We should determine appropriate values.
+  */
+  #define MAX_TABLE_NAME_LEN      64
+  #define MAX_CHAIN_NAME_LEN      64
+  #define MAX_TARGET_NAME_LEN     64
+
+  /* Fwknop custom chain types
+  */
+  enum {
+      FIREWD_INPUT_ACCESS,
+      FIREWD_OUTPUT_ACCESS,
+      FIREWD_FORWARD_ACCESS,
+      FIREWD_DNAT_ACCESS,
+      FIREWD_SNAT_ACCESS,
+      FIREWD_MASQUERADE_ACCESS,
+      NUM_FWKNOP_ACCESS_TYPES  /* Leave this entry last */
+  };
+
+  /* Structure to define an fwknop firewall chain configuration.
+  */
+  struct fw_chain {
+      int     type;
+      char    target[MAX_TARGET_NAME_LEN];
+      //int     direction;
+      char    table[MAX_TABLE_NAME_LEN];
+      char    from_chain[MAX_CHAIN_NAME_LEN];
+      int     jump_rule_pos;
+      char    to_chain[MAX_CHAIN_NAME_LEN];
+      int     rule_pos;
+      int     active_rules;
+      time_t  next_expire;
+  };
+
+  /* Based on the fw_chain fields (not counting type)
+  */
+  #define FW_NUM_CHAIN_FIELDS 6
+
+  struct fw_config {
+      struct fw_chain chain[NUM_FWKNOP_ACCESS_TYPES];
+      char            fw_command[MAX_PATH_LEN];
+
+      /* Flag for setting destination field in rule
+      */
+      unsigned char   use_destination;
+  };
+
+#elif FIREWALL_IPTABLES
   /* --DSS XXX: These are arbitrary. We should determine appropriate values.
   */
   #define MAX_TABLE_NAME_LEN      64
@@ -383,9 +552,9 @@ typedef struct acc_stanza
       struct fw_chain chain[NUM_FWKNOP_ACCESS_TYPES];
       char            fw_command[MAX_PATH_LEN];
 
-      /* Flag for iptables SNAT vs. MASQUERADE usage
+      /* Flag for setting destination field in rule
       */
-      unsigned char   use_masquerade;
+      unsigned char   use_destination;
   };
 
 #elif FIREWALL_IPFW
@@ -402,6 +571,7 @@ typedef struct acc_stanza
       time_t            next_expire;
       time_t            last_purge;
       char              fw_command[MAX_PATH_LEN];
+      unsigned char     use_destination;
   };
 
 #elif FIREWALL_PF
@@ -413,6 +583,7 @@ typedef struct acc_stanza
       time_t            next_expire;
       char              anchor[MAX_PF_ANCHOR_LEN];
       char              fw_command[MAX_PATH_LEN];
+      unsigned char     use_destination;
   };
 
 #elif FIREWALL_IPF
@@ -445,6 +616,8 @@ typedef struct spa_data
     char           *spa_message;
     char            spa_message_src_ip[MAX_IPV4_STR_LEN];
     char            pkt_source_ip[MAX_IPV4_STR_LEN];
+    char            pkt_source_xff_ip[MAX_IPV4_STR_LEN];
+    char            pkt_destination_ip[MAX_IPV4_STR_LEN];
     char            spa_message_remain[1024]; /* --DSS FIXME: arbitrary bounds */
     char           *nat_access;
     char           *server_auth;
@@ -469,8 +642,22 @@ typedef struct fko_srv_options
     unsigned char   fw_list;            /* List current firewall rules */
     unsigned char   fw_list_all;        /* List all current firewall rules */
     unsigned char   fw_flush;           /* Flush current firewall rules */
+    unsigned char   key_gen;            /* Generate keys and exit */
+    unsigned char   exit_after_parse_config; /* Parse config and exit */
+    unsigned char   exit_parse_digest_cache; /* Parse digest cache and exit */
+
+    /* Operational flags
+    */
     unsigned char   test;               /* Test mode flag */
+    unsigned char   afl_fuzzing;        /* SPA pkts from stdin for AFL fuzzing */
     unsigned char   verbose;            /* Verbose mode flag */
+    unsigned char   enable_udp_server;  /* Enable UDP server mode */
+    unsigned char   enable_nfq_capture; /* Enable Netfilter Queue capture mode */
+    unsigned char   enable_fw;          /* Command modes by themselves don't
+                                           need firewall support. */
+
+    unsigned char   firewd_disable_check_support; /* Don't use firewall-cmd ... -C */
+    unsigned char   ipt_disable_check_support;    /* Don't use iptables -C */
 
     /* Flag for permitting SPA packets regardless of directionality test
      * w.r.t. the sniffing interface.  This can sometimes be useful for SPA
@@ -483,6 +670,13 @@ typedef struct fko_srv_options
     int             tcp_server_pid;
     int             lock_fd;
 
+    /* Values used in --key-gen mode only
+    */
+    char key_gen_file[MAX_PATH_LEN];
+    int  key_len;
+    int  hmac_key_len;
+    int  hmac_type;
+
 #if USE_FILE_CACHE
     struct digest_cache_list *digest_cache;   /* In-memory digest cache list */
 #endif
@@ -500,24 +694,55 @@ typedef struct fko_srv_options
     */
     char           *config[NUMBER_OF_CONFIG_ENTRIES];
 
+    /* Data elements that are derived from configuration entries - avoids
+     * calling strtol_wrapper() after the config is parsed.
+    */
+    unsigned short tcpserv_port;
+    unsigned short udpserv_port;
+    int            udpserv_select_timeout;
+    int            rules_chk_threshold;
+    int            pcap_loop_sleep;
+    int            pcap_dispatch_count;
+    int            max_sniff_bytes;
+    int            max_spa_packet_age;
+
     acc_stanza_t   *acc_stanzas;       /* List of access stanzas */
 
     /* Firewall config info.
     */
     struct fw_config *fw_config;
 
+    /* Rule checking counter - this is for garbage cleanup mode to remove
+     * any rules with an expired timer (even those that may have been
+     * added by a third-party program).
+    */
+    unsigned int check_rules_ctr;
+
+    /* Track external command execution cycles (track source IP, access.conf
+     * stanza number, and instantiation time).
+    */
+    cmd_cycle_list_t *cmd_cycle_list;
+
     /* Set to 1 when messages have to go through syslog, 0 otherwise */
     unsigned char   syslog_enable;
 
 } fko_srv_options_t;
 
-extern fko_srv_options_t options;
-
 /* For cleaning up memory before exiting
 */
 #define FW_CLEANUP          1
 #define NO_FW_CLEANUP       0
-void clean_exit(fko_srv_options_t *opts, unsigned int fw_cleanup_flag, unsigned int exit_status);
+
+/**
+ * \brief Frees all memory and exits
+ *
+ * \param opts Program options
+ * \param fw_cleanup_flag Flag indicates whether firewall needs cleanup
+ * \param exit_status Exit status to return when closing the program
+ *
+ */
+void clean_exit(fko_srv_options_t *opts,
+        unsigned int fw_cleanup_flag, unsigned int exit_status);
 
 #endif /* FWKNOPD_COMMON_H */
 
