@@ -1,13 +1,12 @@
-/*
- *****************************************************************************
+/**
+ * \file server/fwknopd_errors.c
  *
- * File:    errors.c
- *
- * Author:  Damien S. Stuart
- *
- * Purpose: Error message functions for fwknopd
- *
- * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief Error message functions for fwknopd
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -106,4 +105,24 @@ get_errstr(const int err_code)
     return(fwknopd_errstr(err_code));
 }
 
+/* print all server errors (from server/fwknopd_errors.c) to stdout
+*/
+void
+dump_server_errors(void)
+{
+    int i;
+    for (i=SPA_MSG_BAD_DATA; i<=SPA_MSG_ERROR; i++)
+    {
+        fprintf(stdout, "err code: %d, err string: '%s'\n",
+                i, fwknopd_errstr(i));
+    }
+    for (i=FW_RULE_ADD_ERROR; i<=FW_RULE_UNKNOWN_ERROR; i++)
+    {
+        fprintf(stdout, "err code: %d, err string: '%s'\n",
+                i, fwknopd_errstr(i));
+    }
+    fflush(stdout);
+    return;
+}
+
 /***EOF***/
