@@ -1,17 +1,18 @@
-/*
- *****************************************************************************
+/**
+ * \file server/replay_cache.c
  *
- * File:    replay_cache.c
+ * \brief Cache packets for replay attack detection
  *
- * Author:  Damien S. Stuart
- *
- * Purpose: Provides the functions to check for possible replay attacks
+ *          Provides the functions to check for possible replay attacks
  *          by using a cache of previously seen digests.  This cache is a
  *          simple file by default, but can be made to use a dbm solution
  *          (ndbm or gdbm in ndbm compatibility mode) file to store the digest
  *          of a previously received SPA packets.
- *
- * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -93,9 +94,9 @@ rotate_digest_cache_file(fko_srv_options_t *opts)
     log_msg(LOG_INFO, "Rotating digest cache file.");
 
 #if USE_FILE_CACHE
-    new_file = malloc(strlen(opts->config[CONF_DIGEST_FILE])+5);
+    new_file = calloc(1, strlen(opts->config[CONF_DIGEST_FILE])+5);
 #else
-    new_file = malloc(strlen(opts->config[CONF_DIGEST_DB_FILE])+5);
+    new_file = calloc(1, strlen(opts->config[CONF_DIGEST_DB_FILE])+5);
 #endif
 
     if(new_file == NULL)
@@ -134,8 +135,8 @@ rotate_digest_cache_file(fko_srv_options_t *opts)
         );
 #endif /* NO_DIGEST_CACHE */
 
-    if(new_file != NULL)
-        free(new_file);
+    free(new_file);
+    return;
 }
 
 static void
@@ -306,12 +307,12 @@ replay_file_cache_init(fko_srv_options_t *opts)
         src_ip[0] = '\0';
         dst_ip[0] = '\0';
 
-        if(sscanf(line_buf, "%s %hhu %s %hu %s %hu %ld",
-            digest_elm->cache_info.digest,
+        if(sscanf(line_buf, "%64s %hhu %16s %hu %16s %hu %ld",
+            digest_elm->cache_info.digest,  /* %64s, buffer size is MAX_DIGEST_SIZE+1 */
             &(digest_elm->cache_info.proto),
-            src_ip,
+            src_ip,  /* %16s, buffer size is INET_ADDRSTRLEN+1 */
             &(digest_elm->cache_info.src_port),
-            dst_ip,
+            dst_ip,  /* %16s, buffer size is INET_ADDRSTRLEN+1 */
             &(digest_elm->cache_info.dst_port),
             &time_tmp) != 7)
         {
@@ -677,10 +678,16 @@ add_replay_dbm_cache(fko_srv_options_t *opts, char *digest)
 void
 free_replay_list(fko_srv_options_t *opts)
 {
+    struct digest_cache_list *digest_list_ptr = NULL, *digest_tmp = NULL;
+
 #ifdef NO_DIGEST_CACHE
     return;
 #endif
-    struct digest_cache_list *digest_list_ptr = NULL, *digest_tmp = NULL;
+
+#if AFL_FUZZING
+    if(opts->afl_fuzzing)
+        return;
+#endif
 
     if (opts->digest_cache == NULL)
         return;
