@@ -1,11 +1,12 @@
 /**
- * @file    utils.c
+ * \file server/utils.c
  *
- * @author  Damien S. Stuart
- *
- * @brief   General/Generic functions for the fwknop server.
- *
- * Copyright 2010-2013 Damien Stuart (dstuart@dstuart.org)
+ * \brief General/Generic functions for the fwknop server.
+ */
+
+/*  Fwknop is developed primarily by the people listed in the file 'AUTHORS'.
+ *  Copyright (C) 2009-2015 fwknop developers and contributors. For a full
+ *  list of contributors, see the file 'CREDITS'.
  *
  *  License (GNU General Public License):
  *
@@ -28,77 +29,88 @@
 #include "fwknopd_common.h"
 #include "utils.h"
 #include "log_msg.h"
-#include <stdarg.h>
+#include "replay_cache.h"
+#include "config_init.h"
+#include "fw_util.h"
+#include "cmd_cycle.h"
 
-/* Generic hex dump function.
+/* Basic directory/binary checks (stat() and whether the path is actually
+ * a directory or an executable).
 */
-void
-hex_dump(const unsigned char *data, const int size)
+static int
+is_valid_path(const char *path, const int file_type)
 {
-    int ln, i, j = 0;
-    char ascii_str[17] = {0};
-
-    for(i=0; i<size; i++)
+    if(strnlen(path, MAX_PATH_LEN) == MAX_PATH_LEN)
     {
-        if((i % 16) == 0)
-        {
-            printf(" %s\n  0x%.4x:  ", ascii_str, i);
-            memset(ascii_str, 0x0, 17);
-            j = 0;
-        }
-
-        printf("%.2x ", data[i]);
-
-        ascii_str[j++] = (data[i] < 0x20 || data[i] > 0x7e) ? '.' : data[i];
-
-        if(j == 8)
-            printf(" ");
+        log_msg(LOG_ERR, "[-] Provided path is too long");
+        return(0);
     }
 
-    /* Remainder...
-    */
-    ln = strlen(ascii_str);
-    if(ln > 0)
-    {
-        for(i=0; i < 16-ln; i++)
-            printf("   ");
-        if(ln < 8)
-            printf(" ");
-
-        printf(" %s\n\n", ascii_str);
-    }
-}
-
-/* Basic directory checks (stat() and whether the path is actually
- * a directory).
-*/
-int
-is_valid_dir(const char *path)
-{
-#if HAVE_STAT
+#if HAVE_STAT || HAVE_LSTAT
     struct stat st;
 
-    /* If we are unable to stat the given dir, then return with error.
+    /* If we are unable to stat the given path, then return with error.
     */
-    if(stat(path, &st) != 0)
+  #if HAVE_LSTAT /* prefer lstat() to stat() */
+    if(lstat(path, &st) != 0)
     {
-        log_msg(LOG_ERR, "[-] unable to stat() directory: %s: %s",
+        log_msg(LOG_ERR, "[-] unable to lstat() path: %s: %s",
             path, strerror(errno));
         return(0);
     }
-
-    if(!S_ISDIR(st.st_mode))
+  #else
+    if(stat(path, &st) != 0)
+    {
+        log_msg(LOG_ERR, "[-] unable to stat() path: %s: %s",
+            path, strerror(errno));
         return(0);
-#endif /* HAVE_STAT */
+    }
+  #endif
+
+    if(file_type == IS_DIR)
+    {
+        if(!S_ISDIR(st.st_mode))
+            return(0);
+    }
+    else if(file_type == IS_EXE)
+    {
+        if(!S_ISREG(st.st_mode) || ! (st.st_mode & S_IXUSR))
+            return(0);
+    }
+    else if(file_type == IS_FILE)
+    {
+        if(!S_ISREG(st.st_mode))
+            return(0);
+    }
+    else
+        return(0);
+
+#endif /* HAVE_STAT || HAVE_LSTAT */
 
     return(1);
 }
 
+int
+is_valid_dir(const char *path)
+{
+    return is_valid_path(path, IS_DIR);
+}
+
+int
+is_valid_exe(const char *path)
+{
+    return is_valid_path(path, IS_EXE);
+}
+
+int
+is_valid_file(const char *path)
+{
+    return is_valid_path(path, IS_FILE);
+}
+
 int
 verify_file_perms_ownership(const char *file)
 {
-    int res = 1;
-
 #if HAVE_STAT
     struct stat st;
 
@@ -115,10 +127,7 @@ verify_file_perms_ownership(const char *file)
                 "[-] file: %s is not a regular file or symbolic link.",
                 file
             );
-            /* when we start in enforcing this instead of just warning
-             * the user
-            res = 0;
-            */
+            return 0;
         }
 
         if((st.st_mode & (S_IRWXU|S_IRWXG|S_IRWXO)) != (S_IRUSR|S_IWUSR))
@@ -152,13 +161,86 @@ verify_file_perms_ownership(const char *file)
         {
             log_msg(LOG_ERR, "[-] stat() against file: %s returned: %s",
                 file, strerror(errno));
-            res = 0;
+            return 0;
         }
     }
 
 #endif
 
-    return res;
+    return 1;
+}
+
+void
+truncate_partial_line(char *str)
+{
+    int i, have_newline=0;
+
+    if(str != NULL && str[0] != 0x0)
+    {
+        for (i=0; i < strlen(str); i++)
+        {
+            if(str[i] == 0x0a)
+            {
+                have_newline = 1;
+                break;
+            }
+        }
+
+        /* Don't zero out any data unless there is at least
+         * one newline
+        */
+        if(have_newline)
+        {
+            for (i=strlen(str)-1; i > 0; i--)
+            {
+                if(str[i] == 0x0a)
+                    break;
+                str[i] = 0x0;
+            }
+        }
+    }
+    return;
+}
+
+/* Simple test to see if a string only contains digits
+*/
+int
+is_digits(const char * const str)
+{
+    int i;
+    if (str != NULL && str[0] != 0x0)
+    {
+        for (i=0; i<strlen(str); i++)
+        {
+            if(!isdigit(str[i]))
+                return 0;
+            i++;
+        }
+    }
+    return 1;
+}
+
+void
+clean_exit(fko_srv_options_t *opts, unsigned int fw_cleanup_flag, unsigned int exit_status)
+{
+#if HAVE_LIBFIU
+    if(opts->config[CONF_FAULT_INJECTION_TAG] != NULL)
+    {
+        fiu_disable(opts->config[CONF_FAULT_INJECTION_TAG]);
+    }
+#endif
+
+    if(!opts->test && opts->enable_fw && (fw_cleanup_flag == FW_CLEANUP))
+        fw_cleanup(opts);
+
+#if USE_FILE_CACHE
+    free_replay_list(opts);
+#endif
+
+    free_logging();
+    free_cmd_cycle_list(opts);
+    free_configs(opts);
+    exit(exit_status);
 }
 
 /***EOF***/
