@@ -96,6 +96,22 @@
         'fw_rule_created' => $NEW_RULE_REQUIRED,
         'fw_rule_removed' => $NEW_RULE_REMOVED,
     },
+    {
+        'category' => 'Rijndael',
+        'subcategory' => 'client->server OS compatibility',
+        'detail'   => 'v2.6.3 OpenBSD-5.5',
+        'function' => \&os_compatibility,
+        'no_ip_check' => 1,
+        'pkt' =>
+            '+Cf1jjCWmISEhw9vQgWnbmqOtHl2rJ0398ILuJSdSHOTnnbkK65sBikWyIEmj4' .
+            'K13ISVG+z7k/bA5+r7Wo1boe02DLjTzkHjryzC8NiWCAUJZoyJu345/eGd+9Dd' .
+            'K1InVWjQFqwC9g8V7cGCogZLAAPqyRC7ZnMvU',
+        'server_positive_output_matches' => [qr/with expire time/],
+        'fwknopd_cmdline' => "$fwknopdCmd -c $cf{'disable_aging'} -a $cf{'def_access'} " .
+            "-d $default_digest_file -p $default_pid_file $intf_str",
+        'fw_rule_created' => $NEW_RULE_REQUIRED,
+        'fw_rule_removed' => $NEW_RULE_REMOVED,
+    },
 
     {
         'category' => 'Rijndael',
@@ -182,6 +198,24 @@
         'fw_rule_created' => $NEW_RULE_REQUIRED,
         'fw_rule_removed' => $NEW_RULE_REMOVED,
     },
+    {
+        'category' => 'Rijndael+HMAC',
+        'subcategory' => 'client->server OS compatibility',
+        'detail'   => 'v2.6.3 FreeBSD10',
+        'function' => \&os_compatibility,
+        'no_ip_check' => 1,
+        'pkt' =>
+            '87ujK1b1aOKxWWI3IWAYUhhimid7In1SQO60K3Kz3BWouDAHVJrthrx8qZbOoNubB' .
+            'nOT06p9lTWhkKirgMRySEFBQ9ypGPYnraysdlqmYuRDJ3ZIL0IWaJywgVPDJlxxPu' .
+            'I8MD6wm4bHHJ8BkSeWqix97DnU215iUkPlD/ks85jHMD5EarLcijkV5OBwilCcVsD' .
+            'j7i6q0PKQ',
+        'server_positive_output_matches' => [qr/with expire time/],
+        'fwknopd_cmdline' => "$fwknopdCmd -c $cf{'disable_aging'} -a $cf{'hmac_access'} " .
+            "-d $default_digest_file -p $default_pid_file $intf_str",
+        'fw_rule_created' => $NEW_RULE_REQUIRED,
+        'fw_rule_removed' => $NEW_RULE_REMOVED,
+    },
+
     {
         'category' => 'Rijndael+HMAC',
         'subcategory' => 'client->server OS compatibility',
@@ -216,6 +250,24 @@
         'fw_rule_created' => $NEW_RULE_REQUIRED,
         'fw_rule_removed' => $NEW_RULE_REMOVED,
     },
+    {
+        'category' => 'Rijndael+HMAC',
+        'subcategory' => 'client->server OS compatibility',
+        'detail'   => 'v2.6.3 OpenBSD5.5',
+        'function' => \&os_compatibility,
+        'no_ip_check' => 1,
+        'pkt' =>
+            '9/sg0eyMETZhbucySDpiApzeCT5QM02kMzdDqOL8VxsR3/sPHwIW3i5F7RS1v4CI' .
+            'kOTdBojvY9o/z0xCmMOpmmyCy0eA8ns4OAl4TKq3NlZCYKPeOgHq80wGB2C0XcEz' .
+            '3hy0QW1sfETjHH/plAlInptHExeyh11PAv1Ef7nMhoz7LGoht867nBKW+Kf9NT54' .
+            'U6ksmbKjeib4',
+        'server_positive_output_matches' => [qr/with expire time/],
+        'fwknopd_cmdline' => "$fwknopdCmd -c $cf{'disable_aging'} -a $cf{'hmac_access'} " .
+            "-d $default_digest_file -p $default_pid_file $intf_str",
+        'fw_rule_created' => $NEW_RULE_REQUIRED,
+        'fw_rule_removed' => $NEW_RULE_REMOVED,
+    },
+
     {
         'category' => 'Rijndael+HMAC',
         'subcategory' => 'client->server OS compatibility',
@@ -233,4 +285,116 @@
         'fw_rule_created' => $NEW_RULE_REQUIRED,
         'fw_rule_removed' => $NEW_RULE_REMOVED,
     },
+
+    ### tests for Jonathan Bennett's Fwknop2 Android app. All commit hashes
+    ### are from: https://github.com/oneru/Fwknop2
+    {
+        'category' => 'Rijndael+HMAC',
+        'subcategory' => 'Android compatibility',
+        'detail'   => 'Fwknop2 (commit 81d4b2f6)',
+        'function' => \&os_compatibility,
+        'no_ip_check' => 1,
+        'pkt' =>
+                '/Tq4' .
+                '6RmmFdCNJOFkesif' .
+                'TKvHZQ2tuSjsPq5f' .
+                'MZqGZVRnd3SFOIeI' .
+                'nw/AId6lRxbhvO1v' .
+                'jNNgJt492yR4gmVZ' .
+                '1LwMKx8zmLVeqV0J' .
+                '+wCWU4ZjNf2FKOlE' .
+                'jvEDFmLIPCqZqli3' .
+                'P4hwUjC+jn0Pkh23' .
+                'GV0uZMm8+Q2k8xVb' .
+                'oZXaWie2hJOUK+oa' .
+                'QxtFZWOUOCTG05oS' .
+                'ZWDQgJrc',
+        'fwknopd_cmdline' => "$fwknopdCmd -c $cf{'disable_aging'} -a $cf{'hmac_android_access'} " .
+            "-d $default_digest_file -p $default_pid_file $intf_str",
+        'server_positive_output_matches' => [qr/with expire time/],
+        'fw_rule_created' => $NEW_RULE_REQUIRED,
+        'fw_rule_removed' => $NEW_RULE_REMOVED,
+    },
+    {
+        'category' => 'Rijndael+HMAC',
+        'subcategory' => 'Android compatibility',
+        'detail'   => 'Fwknop2 (Beta1)',
+        'function' => \&os_compatibility,
+        'no_ip_check' => 1,
+        'pkt' =>
+                '8QlE' .
+                'OMGvx+FgQZ1Ipdrp' .
+                '9G++fRv/ljKwAduX' .
+                'Lj8d3rtPEHXlR+F/' .
+                '2rVTUgumPMVU8NYf' .
+                'VR0wJBHO6/9wbvbB' .
+                'ePlmwoSeVBXYT3N0' .
+                'jsuaH6YUG/9udyfc' .
+                'hQTYVXbtEnAxtScu' .
+                'dZP5eWXPthoL1Jl8' .
+                'HrEF78mVjjINU7pO' .
+                'B4jEP5Hd+1RWzgv+' .
+                'NH9sm2aUVzeB3Iux' .
+                'QnaMhunU',
+        'fwknopd_cmdline' => "$fwknopdCmd -c $cf{'disable_aging'} -a $cf{'hmac_android_access'} " .
+            "-d $default_digest_file -p $default_pid_file $intf_str",
+        'server_positive_output_matches' => [qr/with expire time/],
+        'fw_rule_created' => $NEW_RULE_REQUIRED,
+        'fw_rule_removed' => $NEW_RULE_REMOVED,
+    },
+    {
+        'category' => 'Rijndael',
+        'subcategory' => 'Android compatibility',
+        'detail'   => 'Fwknop2-v1.0-RC.2',
+        'function' => \&os_compatibility,
+        'no_ip_check' => 1,
+        'pkt' =>
+                '8noA' .
+                'VjCwHjmqXgqngHY/' .
+                'JzHi3Bd9LB4E6eeC' .
+                '5dyVG7PmsOjmLITh' .
+                'LLNAj3JPbzBodUYS' .
+                'h/g5/rY88m/EtFhy' .
+                'eSACzRuRC+LVYuI2' .
+                'M/2LmCFKNpsq0jAm' .
+                'BuUv3K9j9j4czpwG' .
+                'FJZNseZpOc4oS8ch' .
+                'VKumQJF6b1AQ1w7H' .
+                'BVFuhp6gQEJifs7o' .
+                '03',
+        'fwknopd_cmdline' => "$fwknopdCmd -c $cf{'disable_aging'} -a $cf{'def_access'} " .
+            "-d $default_digest_file -p $default_pid_file $intf_str",
+        'server_positive_output_matches' => [qr/with expire time/],
+        'fw_rule_created' => $NEW_RULE_REQUIRED,
+        'fw_rule_removed' => $NEW_RULE_REMOVED,
+    },
+    {
+        'category' => 'Rijndael+HMAC',
+        'subcategory' => 'Android compatibility',
+        'detail'   => 'F-Droid Fwknop2 2015-06-16',
+        'function' => \&os_compatibility,
+        'no_ip_check' => 1,
+        'pkt' =>
+                '9OBA' .
+                'YI4pRmsVz1AXyO1M' .
+                'zkbx0RdUx6C+zHXM' .
+                '9aegQZFkYl0agWNI' .
+                'Wqh2kbIlbUvqq1NG' .
+                '47cp3AyccBG4+6SX' .
+                'Qpe4G9JvDgtcYc6D' .
+                'QaG1pT48zi+6BDvn' .
+                'V3K2eojdaPTIIdFR' .
+                'n75c278iCdxue/WW' .
+                '54iK1n07GtNSl8xy' .
+                'jTBiC4dlEfGpYbM7' .
+                'FIGmIciVZo3PZOxt' .
+                'apdF8Ml9SRwHwka/' .
+                'pCS+3hKnjbUOk',
+        'fwknopd_cmdline' => "$fwknopdCmd -c $cf{'disable_aging'} -a $cf{'hmac_android_fdroid_access'} " .
+            "-d $default_digest_file -p $default_pid_file $intf_str",
+        'server_positive_output_matches' => [qr/with expire time/],
+        'fw_rule_created' => $NEW_RULE_REQUIRED,
+        'fw_rule_removed' => $NEW_RULE_REMOVED,
+    },
+
 );
